/****************************************************************************
 * QSmithTool :                                                             *
 *   Copyright (C) 2015 by Sébastien Lissarre                               *
 *   sebastien.lissarre@gmail.com                                           *
 *                                                                          *
 * Smith Chart drawing base :                                               *
 *   Copyright (C) 2008 by Lisandro Damián Nicanor Pérez Meyer              *
 *   perezmeyer@gmail.com                                                   *
 *                                                                          *
 * Bezier curve :                                                           *
 *   Copyright (C) by Oleg V. Polikarpotchkin                               *
 *   found on http://www.codeproject.com/                                   *
 *                                                                          *
 *  This file is part of QSmithTool.                                        *
 *                                                                          *
 *  QSmithTool is free software: you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  QSmithTool is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with QSmithTool.  If not, see <http://www.gnu.org/licenses/>.     *
 ***************************************************************************/


#ifndef BEZIERCURVE_H
#define BEZIERCURVE_H

#include <QVector>
#include <QString>
#include <QDebug>

#include "rfFctsLibrary.h"

/**
This class calculates bezier curve control points from a vector of knot points
*/
class BezierCurve
{

public:
    BezierCurve();

    /**
       @brief Calculate the two control points between each points of a vector of knot points
       @param knots - Input Knot Bezier spline points
       @return 1 if an error occured, 0 if ok
    */
    int calcCurveControlPoints(CurvePointsList knots);

    /**
       @brief Get the results for the point number 'index'
       @param index - point number
       @return the control point (first or second control point)
    */
    QPointF getFirstControlPoint(int index);
    QPointF getSecondControlPoint(int index);

private:
    /**
        @brief Solves a tridiagonal system for one of coordinates (x or y) of first Bezier control points.
        @param rhs Right hand side vector.
        @return Solution vector.
    */
    QVector<double> calcFirstControlPoints(QVector<double> rhs);

    /// Contains all the data points
    CurvePointsList m_firstControlPoints;
    CurvePointsList m_secondControlPoints;

};

#endif // BEZIERCURVE_H
