/****************************************************************************
 * QSmithTool :                                                             *
 *   Copyright (C) 2015 by Sébastien Lissarre                               *
 *   sebastien.lissarre@gmail.com                                           *
 *                                                                          *
 * Smith Chart drawing base :                                               *
 *   Copyright (C) 2008 by Lisandro Damián Nicanor Pérez Meyer              *
 *   perezmeyer@gmail.com                                                   *
 *                                                                          *
 *                                                                          *
 *  This file is part of QSmithTool.                                        *
 *                                                                          *
 *  QSmithTool is free software: you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  QSmithTool is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with QSmithTool.  If not, see <http://www.gnu.org/licenses/>.     *
 ***************************************************************************/

#ifndef SIMULATION_H
#define SIMULATION_H

#include <QProcess>
#include <QFileInfo>
#include <QDir>
#include <QElapsedTimer>

#include "circuit.h"
#include "smithchart.h"

//#define _SIM_QDEBUG // if defined, all qDebug() msg are activated (slow a lot calculation loops)

class circuitSimulator
{

public:
    circuitSimulator() {}

    /**
    @brief Start the simulation for a given frequency, start point and circuit
    @param _startPoints is the list of start point (with their frequency) in the Smith Chart to simulate the circuit
    @param circuit is the list containing all the components and their parameters
    @param _simQuality is the quality needed for simulation (for ex QUICK_SIM increase calc speed for real time move of a start point by example)
    @return None
    */
    void startSimulation(const QVector<complexFAndFreq> &_startPoints, Circuit &_circuit, const SimQuality _simQuality);

protected :
    /**
    @brief Generate a scale a component values to simulate and call ZinPointCalculation() for each given start point / frequency, component value generated and component type
    @param Almost the sames as startSimulation() parameters
    @return None
    */
    void circuitSimulation(const complexFAndFreq _startPoint, int _numStartPoint, Circuit &_circuit, const SimQuality _simQuality);

    /**
     * @brief Generate a linear scale for all the component value (ex : for a resistor 10ohm => 1,2,3,4,...,10)
     * @param start -> scale start value
     * @param stop -> scal stop value
     * @param nbVal -> number of value in the scale
     * @param numVal -> which one to return
     * @return the value of index numVal
     */
    double linearScale(const double start, const double stop, int nbVal, int numVal);
    /**
     * @brief Generate a non linear scale (not far from log(x), used for parallel inductor for example, a linear scale will give non regulary spacing impedance)
     * @param Quite the same as linearScale()
     */
    double invNonLinearScale(const double lastVal, int nbVal, int numVal);

private:


};

#endif //SIMULATION_H

