/****************************************************************************
 * QSmithTool :                                                             *
 *   Copyright (C) 2015 by Sébastien Lissarre                               *
 *   sebastien.lissarre@gmail.com                                           *
 *                                                                          *
 *                                                                          *
 *  This file is part of QSmithTool.                                        *
 *                                                                          *
 *  QSmithTool is free software: you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  QSmithTool is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with QSmithTool.  If not, see <http://www.gnu.org/licenses/>.     *
 ***************************************************************************/


#ifndef DEFINES
#define DEFINES

#include <complex>
using namespace std;

#define _SOFT_NAME QString("QSmithTool v1.4.0")
#define _PI 3.141592654

/**
 * @brief The simPtsDensity enum, used by each component of the circuit to store the number of point wich will be simulated
 */
enum SimPtsDensity {
    LOW_SIM_POINTS_DENSITY     = 50, // 50 points to simulate
    MEDIUM_SIM_POINTS_DENSITY = 100, // 100
    HIGH_SIM_POINTS_DENSITY    = 200,
    V_HIGH_SIM_POINTS_DENSITY    = 300,
    ULTRA_SIM_POINTS_DENSITY   = 500,
    SMOOTH_RENDER_SIM_POINTS_DENSITY = 5000 // only for particular smooth render
};

/**
 * @brief The SimQuality enum, used to force a quick simulation (=> LOW_SIM_POINTS_DENSITY used), or a high quality sim (=> SMOOTH_RENDER_SIM_POINTS_DENSITY used)
 */
enum SimQuality {
    QUICK_SIM    = 0,
    STANDARD_SIM = 1,
    HQ_SIM       = 2
};

/// complexF is the base type for this Smith Tool, every impedance will be stored with two double
#define complexF std::complex<double>
/// Used to store the circuit in an array of components
#define ComponentsList QVector<Component>
/// Useful to manipulated start impedance points : a complex for the impedance, a double for the simulation frequency of this point
struct complexFAndFreq {
   complexF point;
   double freq;
};
complexFAndFreq complexFAndFreqInitializer();

/// Used to calculate Bezier curves from an array of points
#define CurvePointsList QVector<QPointF>
/// Used to store an array of component values or line length (like 1.2pF, 2.2pF, ...)
#define CompParametersList QVector<double>

#endif // DEFINES

