
#include "dialogParametersDiscrete.h"

dialogParametersDiscrete::dialogParametersDiscrete(QWidget *parent, ComponentType type, CompParametersList listParams)
    : QDialog(parent)
{
    QVector<QString> defaultLabelText;
    QVector<QString> defaultValueText;
    QString title;

    // Discrete component : one parameters (maybe 2 for parasitics in a futur release)
    nbParameters=1;
    defaultLabelText.resize(nbParameters);
    if ((type==CAPA_S)||(type==CAPA_P))
        defaultLabelText[0]="Component value (pF) : ";
    else if ((type==SELF_S)||(type==SELF_P))
        defaultLabelText[0]="Component value (nH) : ";
    else if ((type==RESIST_S)||(type==RESIST_P))
        defaultLabelText[0]="Component value (Ω) : ";
    defaultValueText.resize(nbParameters);
    defaultValueText[0]=doubleToQString(listParams[0]);
    title = "Component parameter edit";


    labelsParam = new QVector<QLabel*>;
    lineEditsParam = new QVector<QLineEdit*>;
    for (int i=0 ; i<nbParameters ; i++)
    {
        labelsParam->append(new QLabel(defaultLabelText[i]));
        lineEditsParam->append(new QLineEdit(defaultValueText[i]));

        connect(lineEditsParam->at(i), SIGNAL(textChanged(const QString &)),
                this, SLOT(enableOkButton()));
    }

    okButton = new QPushButton(tr("&Ok"));
    okButton->setDefault(true);

    closeButton = new QPushButton(tr("&Quitter"));

    connect(okButton, SIGNAL(clicked()),
            this, SLOT(okClicked()));
    connect(closeButton, SIGNAL(clicked()),
            this, SLOT(close()));


    QVBoxLayout *paramsLayout = new QVBoxLayout;
    for (int i=0 ; i<nbParameters ; i++)
    {
        QHBoxLayout *tmpHLayout = new QHBoxLayout;
        tmpHLayout->addWidget(labelsParam->at(i));
        tmpHLayout->addWidget(lineEditsParam->at(i));
        paramsLayout->addLayout(tmpHLayout);
    }

    QVBoxLayout *buttonsLayout = new QVBoxLayout;
    buttonsLayout->addWidget(okButton);
    buttonsLayout->addWidget(closeButton);
    buttonsLayout->addStretch();

    QHBoxLayout *mainLayout = new QHBoxLayout;
    mainLayout->addLayout(paramsLayout);
    mainLayout->addLayout(buttonsLayout);
    setLayout(mainLayout);

    setWindowTitle(title);
    setFixedHeight(sizeHint().height());

    values.clear();

    // Prepare selection to modify line length or comp value
    lineEditsParam->at(0)->selectAll();
}

void dialogParametersDiscrete::okClicked()
{
    values.resize(nbParameters);
    for (int i=0; i<nbParameters ; i++)
    {
        QString text = lineEditsParam->at(i)->text();
        values[i] = text.toDouble();
    }
    accept();
}

void dialogParametersDiscrete::enableOkButton()
{
    bool ok=true;
    double val=0.0;

    for (int i=0; i<nbParameters ; i++)
    {
        val = lineEditsParam->at(i)->text().toDouble(&ok);

        if (ok==false || val<0.0)
        {
            okButton->setEnabled(false);
            return;
        }
    }

    okButton->setEnabled(true);
}

QVector<double> & dialogParametersDiscrete::getValues()
{
    return values;
}


