
#include "dialogParametersLine.h"

dialogParametersLine::dialogParametersLine(QWidget *parent, CompParametersList listParams)
    : QDialog(parent)
{
    // Impedance defined line
    modelLineGroupBox = new QGroupBox(tr("Impedance defined line"));
    labelParamLength = new QLabel(tr("Physical length (mm) : "));
    labelParamImpedance = new QLabel(tr("Line impedance (ohm) : "));
    labelParamPermitivity = new QLabel(tr("Relative permitivity : "));
    labelParamAttenuation = new QLabel(tr("α dB/m (phys. length) : "));
    setAllLinesPushButton = new QPushButton(tr("Set to all lines"));
    lineEditParamLength = new QLineEdit(doubleToQString(listParams[0]));
    lineEditParamImpedance = new QLineEdit(doubleToQString(listParams[1]));
    lineEditParamPermitivity = new QLineEdit(doubleToQString(listParams[2]));
    lineEditParamAttenuation = new QLineEdit(doubleToQString(listParams[3]));

    QHBoxLayout *tmpHLayout1 = new QHBoxLayout;
    tmpHLayout1->addWidget(labelParamLength);
    tmpHLayout1->addWidget(lineEditParamLength);
    QHBoxLayout *tmpHLayout2 = new QHBoxLayout;
    tmpHLayout2->addWidget(labelParamImpedance);
    tmpHLayout2->addWidget(lineEditParamImpedance);
    QHBoxLayout *tmpHLayout3a = new QHBoxLayout;
    tmpHLayout3a->addWidget(labelParamPermitivity);
    tmpHLayout3a->addWidget(lineEditParamPermitivity);
    tmpHLayout3a->addWidget(setAllLinesPushButton);
    QHBoxLayout *tmpHLayout3b = new QHBoxLayout;
    tmpHLayout3b->addWidget(labelParamAttenuation);
    tmpHLayout3b->addWidget(lineEditParamAttenuation);

    connect(lineEditParamLength, SIGNAL(textChanged(const QString &)),
                this, SLOT(enableOkButton()));
    connect(lineEditParamImpedance, SIGNAL(textChanged(const QString &)),
                this, SLOT(enableOkButton()));
    connect(lineEditParamPermitivity, SIGNAL(textChanged(const QString &)),
                this, SLOT(enableOkButton()));
    connect(lineEditParamAttenuation, SIGNAL(textChanged(const QString &)),
            this, SLOT(enableOkButton()));
    connect(setAllLinesPushButton, SIGNAL(clicked()),
                this, SLOT(setAllLinesButton()));


    // Physical parameters defined line
    physicalLineGroupBox = new QGroupBox(tr("Physical line"));
    labelParamW = new QLabel(tr("Line width (mm) : "));
    lineEditParamW = new QLineEdit(doubleToQString(0.6));
    labelParamS = new QLabel(tr("Gap width (mm) : "));
    lineEditParamS = new QLineEdit(doubleToQString(0.4));
    labelParamL = new QLabel(tr("Line length (mm) : "));
    lineEditParamL = new QLineEdit(doubleToQString(10.0));
    labelParamH = new QLabel(tr("Heigth of substrate (mm) : "));
    lineEditParamH = new QLineEdit(doubleToQString(0.254));

    QHBoxLayout *tmpHLayout4 = new QHBoxLayout;
    tmpHLayout4->addWidget(labelParamW);
    tmpHLayout4->addWidget(lineEditParamW);
    QHBoxLayout *tmpHLayout5 = new QHBoxLayout;
    tmpHLayout5->addWidget(labelParamS);
    tmpHLayout5->addWidget(lineEditParamS);
    QHBoxLayout *tmpHLayout6 = new QHBoxLayout;
    tmpHLayout6->addWidget(labelParamL);
    tmpHLayout6->addWidget(lineEditParamL);
    QHBoxLayout *tmpHLayout7 = new QHBoxLayout;
    tmpHLayout7->addWidget(labelParamH);
    tmpHLayout7->addWidget(lineEditParamH);

    connect(lineEditParamW, SIGNAL(textChanged(const QString &)),
                this, SLOT(enableOkButton()));

    // Ok&cancel buttons
    okButton = new QPushButton(tr("&Ok"));
    okButton->setDefault(true);

    closeButton = new QPushButton(tr("&Quitter"));

    connect(okButton, SIGNAL(clicked()),
            this, SLOT(okClicked()));
    connect(closeButton, SIGNAL(clicked()),
            this, SLOT(close()));


    radioButtonGroup = new QButtonGroup;

    modelLineRadioButton = new QRadioButton(tr("Model defined line"));
    QVBoxLayout *paramsLayout1 = new QVBoxLayout;
    paramsLayout1->addLayout(tmpHLayout1);
    paramsLayout1->addLayout(tmpHLayout2);
    paramsLayout1->addLayout(tmpHLayout3a);
    paramsLayout1->addLayout(tmpHLayout3b);
    modelLineGroupBox->setLayout(paramsLayout1);

    physicalLineRadioButton = new QRadioButton(tr("Physical line (future release)"));
    QVBoxLayout *paramsLayout2 = new QVBoxLayout;
    paramsLayout2->addLayout(tmpHLayout4);
    paramsLayout2->addLayout(tmpHLayout5);
    paramsLayout2->addLayout(tmpHLayout6);
    paramsLayout2->addLayout(tmpHLayout7);
    physicalLineGroupBox->setLayout(paramsLayout2);

    QVBoxLayout *buttonsLayout = new QVBoxLayout;
    buttonsLayout->addWidget(okButton);
    buttonsLayout->addWidget(closeButton);
    buttonsLayout->addStretch();

    QVBoxLayout *mainParamsLayout = new QVBoxLayout;
    mainParamsLayout->addWidget(modelLineRadioButton);
    mainParamsLayout->addWidget(modelLineGroupBox);
    mainParamsLayout->addWidget(physicalLineRadioButton);
    mainParamsLayout->addWidget(physicalLineGroupBox);
    radioButtonGroup->addButton(modelLineRadioButton);
    radioButtonGroup->addButton(physicalLineRadioButton);
    QHBoxLayout *mainLayout = new QHBoxLayout;
    mainLayout->addLayout(mainParamsLayout);
    mainLayout->addLayout(buttonsLayout);
    setLayout(mainLayout);

    setWindowTitle(tr("Line parameters edit"));
    setFixedHeight(sizeHint().height());

    values.clear();

    // For next release
    physicalLineRadioButton->setEnabled(false);
    physicalLineGroupBox->setEnabled(false);
    physicalLineGroupBox->setStyleSheet("QGroupBox::title{ color: gray }");

    // Prepare selection to modify line length or comp value
    modelLineRadioButton->setChecked(true);
    lineEditParamLength->setFocus(Qt::ActiveWindowFocusReason);
    lineEditParamLength->selectAll();

}

void dialogParametersLine::okClicked()
{
    values.resize(4);

    values[0] = lineEditParamLength->text().toDouble(); // line length
    values[1] = lineEditParamImpedance->text().toDouble(); // line impedance
    values[2] = lineEditParamPermitivity->text().toDouble(); // relative permivity
    values[3] = lineEditParamAttenuation->text().toDouble(); // relative permivity

    accept();
}

void dialogParametersLine::enableOkButton()
{
    bool ok=true;
    double val=0.0;

    val = lineEditParamLength->text().toDouble(&ok);
    if (ok==false || val<0.0)
    {
        okButton->setEnabled(false);
        return;
    }
    val = lineEditParamImpedance->text().toDouble(&ok);
    if (ok==false || val<0.0)
    {
        okButton->setEnabled(false);
        return;
    }
    val = lineEditParamPermitivity->text().toDouble(&ok);
    if (ok==false || val<0.0)
    {
        okButton->setEnabled(false);
        return;
    }
    val = lineEditParamAttenuation->text().toDouble(&ok);
    if (ok==false || val<0.0)
    {
        okButton->setEnabled(false);
        return;
    }

    okButton->setEnabled(true);
}

void dialogParametersLine::setAllLinesButton()
{
    bool ok=true;

    double val = lineEditParamPermitivity->text().toDouble(&ok);
    if (ok==true && val>0.0)
    {
        emit(setPermitivityToAllLinesClicked(val));
    }
}

QVector<double> & dialogParametersLine::getValues()
{
    return values;
}



