/****************************************************************************
 * QSmithTool :                                                             *
 *   Copyright (C) 2015 by Sébastien Lissarre                               *
 *   sebastien.lissarre@gmail.com                                           *
 *                                                                          *
 * Smith Chart drawing base :                                               *
 *   Copyright (C) 2008 by Lisandro Damián Nicanor Pérez Meyer              *
 *   perezmeyer@gmail.com                                                   *
 *                                                                          *
 *                                                                          *
 *  This file is part of QSmithTool.                                        *
 *                                                                          *
 *  QSmithTool is free software: you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  QSmithTool is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with QSmithTool.  If not, see <http://www.gnu.org/licenses/>.     *
 ***************************************************************************/

#ifndef DIALOGPARAMETERSLINE_H
#define DIALOGPARAMETERSLINE_H

#include <QDialog>
#include <QCheckBox>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGroupBox>
#include <QButtonGroup>
#include <QRadioButton>

#include "rfFctsLibrary.h"


/**
This class instanciate a QDialog to allow user to edit the three parameters of a line.
*/
class dialogParametersLine : public QDialog
{
    Q_OBJECT

public:
    dialogParametersLine(QWidget *parent = 0, CompParametersList listParams = CompParametersList(0));
    QVector<double> & getValues();

private slots:
    void okClicked();
    void enableOkButton();
    void setAllLinesButton();

signals:
    void setPermitivityToAllLinesClicked(double relPerm);

private:
    QButtonGroup *radioButtonGroup;
    QRadioButton *modelLineRadioButton;
    QRadioButton *physicalLineRadioButton;

    QGroupBox *modelLineGroupBox;
    QLabel *labelParamLength;
    QLabel *labelParamImpedance;
    QLabel *labelParamPermitivity;
    QLabel *labelParamAttenuation;
    QLineEdit *lineEditParamLength;
    QLineEdit *lineEditParamImpedance;
    QLineEdit *lineEditParamPermitivity;
    QLineEdit *lineEditParamAttenuation;
    QPushButton *setAllLinesPushButton;

    QGroupBox *physicalLineGroupBox;
    QLabel *labelParamW;
    QLineEdit *lineEditParamW;
    QLabel *labelParamS;
    QLineEdit *lineEditParamS;
    QLabel *labelParamL;
    QLineEdit *lineEditParamL;
    QLabel *labelParamH;
    QLineEdit *lineEditParamH;

    QPushButton *okButton;
    QPushButton *closeButton;

    QVector<double> values;
};

#endif



