
#include "dialogPointsConfig.h"



dialogPointsConfig::dialogPointsConfig(QVector<complexFAndFreq> &currentStartPoints, double _Z0, QWidget *parent)
    : QDialog(parent)
{
    m_impedanceZ0 = _Z0;
    m_defaultFolder=QString();

    QString strItems[7] = {tr("Impedance (cart)"),
                           tr("Impedance (polar)"),
                           tr("Admittance (cart)"),
                           tr("Admittance (polar)"),
                           tr("Reflection coeff/S11/S22 (cart)"),
                           tr("Reflection coeff/S11/S22 (polar mag/°)"),
                           tr("Reflection coeff/S11/S22 (polar dB/°)")
                          };

    itemComboBox = new QComboBox();
    itemComboBox->addItem(strItems[0]);
    itemComboBox->addItem(strItems[1]);
    itemComboBox->addItem(strItems[2]);
    itemComboBox->addItem(strItems[3]);
    itemComboBox->addItem(strItems[4]);
    itemComboBox->addItem(strItems[5]);
    itemComboBox->addItem(strItems[6]);

    okButton = new QPushButton(tr("&Ok"));
    okButton->setDefault(true);

    closeButton = new QPushButton(tr("&Quit"));

    labelInput = new QLabel(tr("Enter the new value : "));
    labelVal1 = new QLabel("Re");
    lineEdit1Val = new QLineEdit(doubleToQString(currentStartPoints.first().point.real()*m_impedanceZ0));
    labelVal2 = new QLabel("Im");
    lineEdit2Val = new QLineEdit(doubleToQString(currentStartPoints.first().point.imag()*m_impedanceZ0));
    labelValFreq = new QLabel("Freq :");
    lineEditValFreq = new QLineEdit(doubleToQString(currentStartPoints.first().freq));
    lineEditValFreq->setMinimumWidth(50);
    labelValUnit = new QLabel("Ghz");
    QSpacerItem *spacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);

    addPointButton = new QPushButton(tr("↓ Add point ↓"));
    addPointButton->setMaximumWidth(100);
    removePointButton = new QPushButton(tr("Remove point"));
    removePointButton->setMaximumWidth(100);
    loadS11FileButton = new QPushButton(tr("Load S11 file..."));
    loadS11FileButton->setMaximumWidth(120);
    loadS22FileButton = new QPushButton(tr("Load S22 file..."));
    loadS22FileButton->setMaximumWidth(120);

    widgetPointsList = new QTableWidget(0,4);
    widgetPointsList->setMinimumWidth(300);
    QStringList columnText = (QStringList() << QString("") << QString("Freq (GHz)") << QString("Re") << QString("Im")  );
    widgetPointsList->setHorizontalHeaderLabels(columnText);
    widgetPointsList->horizontalHeader()->resizeSection(0,20);
    widgetPointsList->horizontalHeader()->resizeSection(1,70);
    widgetPointsList->horizontalHeader()->resizeSection(2,70);
    widgetPointsList->horizontalHeader()->resizeSection(3,70);
    widgetPointsList->verticalHeader()->hide();

    for (int numStartPoint=0 ; (numStartPoint<currentStartPoints.size()) && (numStartPoint<10) ; numStartPoint++) {  // each start points
        widgetPointsList->insertRow(widgetPointsList->rowCount());
        QTableWidgetItem *itemCheckBox = new QTableWidgetItem();
        itemCheckBox->setCheckState(Qt::Checked);
        widgetPointsList->setItem(numStartPoint, 0, itemCheckBox);
        widgetPointsList->setItem(numStartPoint, 1, new QTableWidgetItem(doubleToQString(currentStartPoints[numStartPoint].freq)));
        widgetPointsList->setItem(numStartPoint, 2, new QTableWidgetItem(doubleToQString(currentStartPoints[numStartPoint].point.real()*m_impedanceZ0)));
        widgetPointsList->setItem(numStartPoint, 3, new QTableWidgetItem(doubleToQString(currentStartPoints[numStartPoint].point.imag()*m_impedanceZ0)));
    }

    QHBoxLayout *tmpHLayout1 = new QHBoxLayout;
    tmpHLayout1->addWidget(labelVal1);
    tmpHLayout1->addWidget(lineEdit1Val);
    tmpHLayout1->addWidget(labelVal2);
    tmpHLayout1->addWidget(lineEdit2Val);

    QHBoxLayout *tmpHLayout2 = new QHBoxLayout;
    tmpHLayout2->addWidget(labelValFreq);
    tmpHLayout2->addWidget(lineEditValFreq);
    tmpHLayout2->addWidget(labelValUnit);
    tmpHLayout2->addItem(spacer);

    QHBoxLayout *tmpHLayout3 = new QHBoxLayout;
    tmpHLayout3->addWidget(addPointButton);
    tmpHLayout3->addWidget(removePointButton);

    QVBoxLayout *paramsLayout = new QVBoxLayout;
    paramsLayout->addWidget(itemComboBox);
    paramsLayout->addWidget(labelInput);
    paramsLayout->addLayout(tmpHLayout1);
    paramsLayout->addLayout(tmpHLayout2);
    paramsLayout->addLayout(tmpHLayout3);
    paramsLayout->addWidget(widgetPointsList);

    QVBoxLayout *buttonsLayout = new QVBoxLayout;
    buttonsLayout->addWidget(okButton);
    buttonsLayout->addWidget(closeButton);
    buttonsLayout->addWidget(loadS11FileButton);
    buttonsLayout->addWidget(loadS22FileButton);
    buttonsLayout->addStretch();

    QHBoxLayout *mainLayout = new QHBoxLayout;
    mainLayout->addLayout(paramsLayout);
    mainLayout->addLayout(buttonsLayout);
    setLayout(mainLayout);


    setWindowTitle(tr("Start/target points edit"));
    setFixedHeight(sizeHint().height());

    connect(okButton, SIGNAL(clicked()), this, SLOT(okClicked()));
    connect(closeButton, SIGNAL(clicked()), this, SLOT(closeClicked()));
    connect(loadS11FileButton, SIGNAL(clicked(bool)), this, SLOT(loadS11FileClicked()));
    connect(loadS22FileButton, SIGNAL(clicked(bool)), this, SLOT(loadS22FileClicked()));

    connect(itemComboBox, SIGNAL(currentIndexChanged(int)),
            this, SLOT(comboBoxChanged(int)));
    connect(lineEdit1Val, SIGNAL(textEdited(QString)),
            this, SLOT(lineEditModified()));
    connect(lineEdit2Val, SIGNAL(textEdited(QString)),
            this, SLOT(lineEditModified()));
    connect(lineEditValFreq, SIGNAL(textEdited(QString)),
            this, SLOT(lineEditModified()));
    connect(addPointButton, SIGNAL(clicked()),
            this, SLOT(addPointButtonSlot()));
    connect(removePointButton, SIGNAL(clicked()),
            this, SLOT(removePointButtonSlot()));

    lineEditModified();
}

void dialogPointsConfig::okClicked()
{
    if (countItemCheckedInTable() == 0)
    {
        qDebug() << "Nb items : " << countItemCheckedInTable();
        QMessageBox myMsgBox(QMessageBox::Warning, "Warning", "No points have been selected, do you want to continue?", QMessageBox::Yes|QMessageBox::No);;
        int ret = myMsgBox.exec();
        if (ret == QMessageBox::Yes)
            accept();
    }
    else if (countItemCheckedInTable() > 10)
    {
        QMessageBox myMsgBox(QMessageBox::Warning, "Warning", "More than 10 points have been selected, this will increase the simulation time, do you want to continue?", QMessageBox::Yes|QMessageBox::No);;
        int ret = myMsgBox.exec();
        if (ret == QMessageBox::Yes)
            accept();
    }
    else
        accept();
}

void dialogPointsConfig::closeClicked()
{
    reject();
}

void dialogPointsConfig::loadS11FileClicked()
{
    QString fileName;

    /// Ask filename and line numbers
    if (m_defaultFolder.isEmpty())
        fileName = QFileDialog::getOpenFileName(this,tr("Open .s1p or .s2p file"), QDir::homePath()+QString("/Desktop/"), tr("Touchstone file (*.s1p, *.s2p)"));
    else
        fileName = QFileDialog::getOpenFileName(this,tr("Open .s1p or .s2p file"), m_defaultFolder, tr("Touchstone file (*.s1p, *.s2p)"));

    if (!fileName.isEmpty()) {
        /// Read s2p or s1p file
        qDebug() << "Loading file : " << fileName;
        m_defaultFolder = QFileInfo(fileName).absolutePath();
        if (loadTouchstoneFile(fileName, S11) > 0) { // more than 0 points read successfully
            updateQTableWidgetWithTouchstoneData();
        }
    }
}
void dialogPointsConfig::loadS22FileClicked()
{
    QString fileName;

    /// Ask filename and line numbers
    if (m_defaultFolder.isEmpty())
        fileName = QFileDialog::getOpenFileName(this,tr("Open .s1p or .s2p file"), QDir::homePath()+QString("/Desktop/"), tr("Touchstone file (*.s1p, *.s2p)"));
    else
        fileName = QFileDialog::getOpenFileName(this,tr("Open .s1p or .s2p file"), m_defaultFolder, tr("Touchstone file (*.s1p, *.s2p)"));

    if (!fileName.isEmpty()) {
        /// Read s2p or s1p file
        qDebug() << "Loading file : " << fileName;

        if (loadTouchstoneFile(fileName, S22) > 0) { // more than 0 points read successfully
            updateQTableWidgetWithTouchstoneData();
        }
    }
}


void dialogPointsConfig::updateQTableWidgetWithTouchstoneData()
{
    //int firstPointNumber = QInputDialog::getInt(this, tr("First line"), tr("First point (total ") + QString::number(SParamExtracted.size()) + QString(") : "), 1, 1, SParamExtracted.size(), 1, &ok);
    //int lastPointNumber = QInputDialog::getInt(this, tr("Last line"), tr("Last point (total ") + QString::number(SParamExtracted.size()) + QString(") : "), 1, 1, SParamExtracted.size(), 1, &ok);

    widgetPointsList->setRowCount(0);
    for (int i=0 ; i<SParamExtracted.size()-1 ; i++)
    {
        //qDebug() << "S22 : " << SParamExtracted[firstPointNumber-1+i].freq <<  SParamExtracted[firstPointNumber-1+i].point.real() <<  SParamExtracted[firstPointNumber-1+i].point.imag();
        widgetPointsList->insertRow(widgetPointsList->rowCount());
        QTableWidgetItem *item = new QTableWidgetItem();
        if (SParamExtracted.size()-1 < 10)
            item->setCheckState(Qt::Checked);
        else
            item->setCheckState(Qt::Unchecked);
        widgetPointsList->setItem(i, 0, item);
        widgetPointsList->setItem(i, 1, new QTableWidgetItem(doubleToQString(SParamExtracted[i].freq, 3)));
        widgetPointsList->setItem(i, 2, new QTableWidgetItem(doubleToQString(SParamExtracted[i].point.real())));
        widgetPointsList->setItem(i, 3, new QTableWidgetItem(doubleToQString(SParamExtracted[i].point.imag())));
    }
}


void dialogPointsConfig::comboBoxChanged(int newIndex)
{
    complexF tmpComplex;

    if ((newIndex==0) || (newIndex==2) || (newIndex==4)) // cartesian inputs
    {
        labelVal1->setText("Re");
        labelVal2->setText("Im");
    }
    else if (newIndex==6) // polar reflex coeff in dB input
    {
        labelVal1->setText(tr("Mag (dB)"));
        labelVal2->setText(tr("Angle(°)"));
    }
    else // other polar inputs
    {
        labelVal1->setText(tr("Mag"));
        labelVal2->setText(tr("Angle(°)"));
    }

    // Recalculate the parameters for each mode
    switch(itemComboBox->currentIndex())
    {
        case 0: //  Zin cart => "Zin (cart)"
            lineEdit1Val->setText(doubleToQString(m_lastCorrectInputValue.point.real()));
            lineEdit2Val->setText(doubleToQString(m_lastCorrectInputValue.point.imag()));
            break;

        case 1: // Zin cart => "Zin (polar)"
            tmpComplex = cartToPolar(m_lastCorrectInputValue.point);
            lineEdit1Val->setText(doubleToQString(tmpComplex.real()));
            lineEdit2Val->setText(doubleToQString(tmpComplex.imag()));
            break;

        case 2: // Zin cart => "Yin (cart)"
            lineEdit1Val->setText(doubleToQString(1.0/m_lastCorrectInputValue.point.real()));
            lineEdit2Val->setText(doubleToQString(1.0/m_lastCorrectInputValue.point.imag()));
            break;

        case 3: // Zin cart => "Yin (polar)"
            tmpComplex = complexF(1.0/m_lastCorrectInputValue.point.real(), 1.0/m_lastCorrectInputValue.point.imag());
            tmpComplex = cartToPolar(tmpComplex);
            lineEdit1Val->setText(doubleToQString(tmpComplex.real()));
            lineEdit2Val->setText(doubleToQString(tmpComplex.imag()));
            break;

        case 4: // Zin cart => "Reflection coeff (cart)"
            tmpComplex = ZloadToCoeffReflect(m_lastCorrectInputValue.point, m_impedanceZ0);
            lineEdit1Val->setText(doubleToQString(tmpComplex.real()));
            lineEdit2Val->setText(doubleToQString(tmpComplex.imag()));
            break;

        case 5: // Zin cart => "Reflection coeff (polar Mag/°)"
            tmpComplex = ZloadToCoeffReflectPolar(m_lastCorrectInputValue.point, m_impedanceZ0);
            lineEdit1Val->setText(doubleToQString(tmpComplex.real()));
            lineEdit2Val->setText(doubleToQString(tmpComplex.imag()));
            break;
        case 6: // Zin cart => "Reflection coeff (polar dB/°)"
            tmpComplex = ZloadToCoeffReflectPolar(m_lastCorrectInputValue.point, m_impedanceZ0);
            lineEdit1Val->setText(doubleToQString(10.0*log10(tmpComplex.real())));
            lineEdit2Val->setText(doubleToQString(tmpComplex.imag()));
            break;
    }
}


void dialogPointsConfig::lineEditModified()
{
    complexF tmpCplxCart, tmpCplxPolar, resultConv;
    double tmpFreq;

    // Check frequency is valid
    if (checkFrequencyValid(lineEditValFreq->text(), &tmpFreq)) {
        lineEditValFreq->setStyleSheet("QLineEdit{background-color: white;}");
        m_lastCorrectInputValue.freq = tmpFreq;
        okButton->setEnabled(true);
    }
    else {
        lineEditValFreq->setStyleSheet("QLineEdit{background-color: red;}");
        okButton->setEnabled(false);
    }

    // Check point entered is valid
    switch(itemComboBox->currentIndex())
    {
    case 0: // "Zin (cart)" => Zin cart
        tmpCplxCart = complexF(lineEdit1Val->text().toDouble(), lineEdit2Val->text().toDouble());
        resultConv = tmpCplxCart;
        break;
    case 1: // "Zin (polar)"  => Zin cart
        tmpCplxPolar = complexF(lineEdit1Val->text().toDouble(), lineEdit2Val->text().toDouble());
        resultConv = polarToCart(tmpCplxPolar);
        break;
    case 2: // "Yin (cart)" => Zin cart
        tmpCplxCart = complexF( lineEdit1Val->text().toDouble(), lineEdit2Val->text().toDouble());
        resultConv = 1.0 / tmpCplxCart;
        break;
    case 3: // "Yin (polar)"  => Zin cart
        tmpCplxPolar = complexF(lineEdit1Val->text().toDouble(), lineEdit2Val->text().toDouble());
        tmpCplxCart = polarToCart(tmpCplxPolar);
        resultConv = 1.0 / tmpCplxCart;
        break;
    case 4: // "Reflection coeff (cart)" => Zin cart
        tmpCplxCart = complexF(lineEdit1Val->text().toDouble(), lineEdit2Val->text().toDouble());
        resultConv = coeffReflectToZload(tmpCplxCart, m_impedanceZ0);
        break;
    case 5: // "Reflection coeff (polar Mag/°)" => Zin cart
        tmpCplxPolar = complexF(lineEdit1Val->text().toDouble(), lineEdit2Val->text().toDouble());
        tmpCplxCart = polarToCart(tmpCplxPolar);
        resultConv = coeffReflectToZload(tmpCplxCart, m_impedanceZ0);
        break;
    case 6: // "Reflection coeff (polar dB/°)" => Zin cart
        tmpCplxPolar = complexF(pow(10.0, lineEdit1Val->text().toDouble()/10.0), lineEdit2Val->text().toDouble());
        tmpCplxCart = polarToCart(tmpCplxPolar);
        resultConv = coeffReflectToZload(tmpCplxCart, m_impedanceZ0);
        break;
    }

    if ((checkRealPartImpValid(resultConv.real()))
            && (checkImagPartImpValid(resultConv.imag())))
    {
        lineEdit1Val->setStyleSheet("QLineEdit{background-color: white;}");
        lineEdit2Val->setStyleSheet("QLineEdit{background-color: white;}");
        okButton->setEnabled(true);
        m_lastCorrectInputValue.point = resultConv;
    }
    else
    {
        lineEdit1Val->setStyleSheet("QLineEdit{background-color: red;}");
        lineEdit2Val->setStyleSheet("QLineEdit{background-color: red;}");
        okButton->setEnabled(false);
    }
}


void dialogPointsConfig::addPointButtonSlot()
{
    if ((checkRealPartImpValid(m_lastCorrectInputValue.point.real()))
            && (checkImagPartImpValid(m_lastCorrectInputValue.point.imag()))
            && (checkFrequencyValid(m_lastCorrectInputValue.freq)))
    {
        widgetPointsList->insertRow(widgetPointsList->rowCount());
        QTableWidgetItem *item = new QTableWidgetItem();
        item->setCheckState(Qt::Checked);
        widgetPointsList->setItem(widgetPointsList->rowCount()-1 , 0, item);
        widgetPointsList->setItem(widgetPointsList->rowCount()-1 , 1, new QTableWidgetItem(doubleToQString(m_lastCorrectInputValue.freq)));
        widgetPointsList->setItem(widgetPointsList->rowCount()-1 , 2, new QTableWidgetItem(doubleToQString(m_lastCorrectInputValue.point.real())));
        widgetPointsList->setItem(widgetPointsList->rowCount()-1 , 3, new QTableWidgetItem(doubleToQString(m_lastCorrectInputValue.point.imag())));
    }

}

void dialogPointsConfig::removePointButtonSlot()
{
    int numRowSelected = widgetPointsList->currentRow();
    if (numRowSelected<0)
        numRowSelected=0;
    widgetPointsList->removeRow(numRowSelected);
}


QVector<complexFAndFreq> dialogPointsConfig::getValues()
{
    QVector<complexFAndFreq> listToReturn;
    for (int i=0 ; i<widgetPointsList->rowCount()  ; i++ ) {
        complexFAndFreq myComplexFAndFreq;
        myComplexFAndFreq.freq = widgetPointsList->item(i, 1)->text().toDouble();
        myComplexFAndFreq.point = complexF(widgetPointsList->item(i, 2)->text().toDouble(), widgetPointsList->item(i, 3)->text().toDouble());
        if (widgetPointsList->item(i, 0)->checkState()==Qt::Checked)
            listToReturn.append(myComplexFAndFreq);
    }
    return listToReturn;
}

int dialogPointsConfig::countItemCheckedInTable()
{
    int count=0;

    for (int i=0 ; i<widgetPointsList->rowCount() ; i++ )
    {
        if (widgetPointsList->item(i, 0)->checkState() == Qt::Checked)
            count++;
    }
    return count;
}


int dialogPointsConfig::loadTouchstoneFile(QString _fileName, SP_TO_EXTRACT _spToExtract)
{
    FREQ_UNIT freq_unit = HZ;
    SP_FORMAT sp_format=MA;
    SP_TYPE sp_type=S;
    double ref_impedance=50.0;
    bool loadFailed=false;

    QFile file(_fileName);
    QString line;
    int nbPointsRead=0;
    QStringList lineSplitted;

    if (file.open(QIODevice::ReadOnly))
    {
        QTextStream stream(&file);
        while(!stream.atEnd())
        {
            /// Reading line by line
            line = stream.readLine();
            line = line.simplified().toLower();
            if (!line.isEmpty()) {
                if (line[0] == '#') // config line
                {
                    //qDebug() << "Line n°" << lineNb << " (config) : " << line;
                    lineSplitted = line.split(' ');
                    if (lineSplitted.size() != 6)
                    {
                        qDebug() << "ERROR #01 : Touchstone file format not recognized.";
                        loadFailed = true;
                    }
                    else {
                        if (lineSplitted[1] == "hz") freq_unit=HZ;
                        else if (lineSplitted[1] == "khz") freq_unit=KHZ;
                        else if (lineSplitted[1] == "mhz") freq_unit=MHZ;
                        else if (lineSplitted[1] == "ghz") freq_unit=GHZ;
                        else freq_unit=FREQ_ERR;

                        if (lineSplitted[2] == "s") sp_type=S; else sp_type=TYPE_ERR;

                        if (lineSplitted[3] == "ma") sp_format=MA;
                        else if (lineSplitted[3] == "db") sp_format=DB;
                        else if (lineSplitted[3] == "ri") sp_format=RI;
                        else sp_format=FORMAT_ERR;

                        if (lineSplitted[5].toDouble() != 0.0)
                            ref_impedance = lineSplitted[5].toDouble();
                        else
                            ref_impedance = 0.0;

                        if ((freq_unit==FREQ_ERR) || (sp_type==TYPE_ERR) || (sp_format==FORMAT_ERR) || (ref_impedance==0.0)) {
                            qDebug() << "ERROR #02 : Touchstone file format not recognized.";
                            loadFailed = true;
                        }
                        //else
                        //    qDebug() << "Format :" << freq_unit << sp_type << sp_format << ref_impedance << " -> OK";
                    }
                }
                else if (line[0] == '!') {} // comment
                else // data line
                {
                    //qDebug() << "Line n°" << lineNb << " : " << line;
                    lineSplitted = line.split(' ');

                    if ((_spToExtract == S11) && (lineSplitted.size() < 3)) {
                        qDebug() << "ERROR #03 : Touchstone S11 file format not recognized.";
                        loadFailed = true;
                    }
                    else if ((_spToExtract == S22) && (lineSplitted.size() < 9)) {
                        qDebug() << "ERROR #04 : Touchstone S22 file format not recognized.";
                        loadFailed = true;
                    }
                    else
                    {
                        complexFAndFreq myComplexAndFreq = complexFAndFreqInitializer();

                        myComplexAndFreq.freq = lineSplitted[0].toDouble();
                        if (freq_unit == HZ)
                            myComplexAndFreq.freq = myComplexAndFreq.freq / 1000000000.0; // all in GHz
                        if (freq_unit == KHZ)
                            myComplexAndFreq.freq = myComplexAndFreq.freq / 1000000.0; // all in GHz
                        if (freq_unit == MHZ)
                            myComplexAndFreq.freq = myComplexAndFreq.freq / 1000.0; // all in GHz

                        if (_spToExtract == S11) {
                            myComplexAndFreq.point.real(lineSplitted[1].toDouble());
                            myComplexAndFreq.point.imag(lineSplitted[2].toDouble());
                        } else { // S22
                            myComplexAndFreq.point.real(lineSplitted[7].toDouble());
                            myComplexAndFreq.point.imag(lineSplitted[8].toDouble());
                        }
                        if (sp_format == MA) {
                            myComplexAndFreq.point = polarToCart(myComplexAndFreq.point); // convert MA polar to MA cart
                            myComplexAndFreq.point = coeffReflectToZload(myComplexAndFreq.point, ref_impedance); // convert MA cart to Zload
                        }
                        else if (sp_format == DB) {
                            myComplexAndFreq.point.real( pow(10.0, myComplexAndFreq.point.real() / 20.0)) ; // convert DB polar to MA polar
                            myComplexAndFreq.point = polarToCart(myComplexAndFreq.point); // convert MA ploar to MA cart
                            myComplexAndFreq.point = coeffReflectToZload(myComplexAndFreq.point, ref_impedance); // convert MA cart to Zload
                        }
                        else if (sp_format == RI) {
                            myComplexAndFreq.point = coeffReflectToZload(myComplexAndFreq.point, ref_impedance); // convert MA cart to Zload
                        }

                        SParamExtracted.append(myComplexAndFreq);
                        nbPointsRead++;
                    }
                }
            }
        }

        file.close();
    }

    if (loadFailed)
        return -1;
    else
        return nbPointsRead;
}






