/****************************************************************************
 * QSmithTool :                                                             *
 *   Copyright (C) 2015 by Sébastien Lissarre                               *
 *   sebastien.lissarre@gmail.com                                           *
 *                                                                          *
 * Smith Chart drawing base :                                               *
 *   Copyright (C) 2008 by Lisandro Damián Nicanor Pérez Meyer              *
 *   perezmeyer@gmail.com                                                   *
 *                                                                          *
 *                                                                          *
 *  This file is part of QSmithTool.                                        *
 *                                                                          *
 *  QSmithTool is free software: you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  QSmithTool is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with QSmithTool.  If not, see <http://www.gnu.org/licenses/>.     *
 ***************************************************************************/


#ifndef DIALOGPOINTSCONF_H
#define DIALOGPOINTSCONF_H

#include <QDialog>
#include <QCheckBox>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QComboBox>
#include <QMessageBox>
#include <QTableWidget>
#include <QHeaderView>

#include <QFileDialog>
#include <QInputDialog>
#include <cmath>

#include "rfFctsLibrary.h"

enum SP_TO_EXTRACT { S11, S22 };
enum FREQ_UNIT { HZ, KHZ, MHZ, GHZ, FREQ_ERR };
enum SP_TYPE { S, TYPE_ERR };
enum SP_FORMAT { MA, DB, RI, FORMAT_ERR };


/**
This class instanciate a QDialog to allow user to edit start points in the Smith Chart
*/
class dialogPointsConfig : public QDialog
{
    Q_OBJECT

public:
    dialogPointsConfig(QVector<complexFAndFreq> &currentStartPoints, double Z0, QWidget *parent = 0);
    QVector<complexFAndFreq> getValues();


private slots:
    /// "Ok" button clicked
    void okClicked();
    /// "Close" button clicked
    void closeClicked();
    /// "S11" button clicked
    void loadS11FileClicked();
    /// "S22" button
    void loadS22FileClicked();
    /// Type of value entered (reflect coeff, impedance, ...)
    void comboBoxChanged(int newIndex);
    void lineEditModified();
    /// Add/remove point in table
    void addPointButtonSlot();
    void removePointButtonSlot();
    int countItemCheckedInTable();

private:
    /**
     * @brief loadTouchstoneFile take a fileName string and open the corresponding touchstone file to extract S11 or S22 from it
     * @param _fileName path string
     * @param _spToExtract S11 or S22
     * @return -1 if failed, number of points extracted if succeed to read
     */
    int loadTouchstoneFile(QString _fileName, SP_TO_EXTRACT _spToExtract);
    void updateQTableWidgetWithTouchstoneData();

    QComboBox *itemComboBox;
    QLabel *labelInput;
    QLabel *labelVal1;
    QLineEdit *lineEdit1Val;
    QLabel *labelVal2;
    QLineEdit *lineEdit2Val;
    QLabel *labelValFreq;
    QLineEdit *lineEditValFreq;
    QLabel *labelValUnit;
    QPushButton *okButton;
    QPushButton *closeButton;

    QPushButton *addPointButton;
    QPushButton *removePointButton;

    QPushButton *loadS11FileButton;
    QPushButton *loadS22FileButton;

    QTableWidget *widgetPointsList;

    complexFAndFreq m_lastCorrectInputValue; /// Stocked in cartesian unormalized impedance, last point entered by user

    double m_impedanceZ0;
    QVector< complexFAndFreq > SParamExtracted;

    QString m_defaultFolder;
};

#endif



