
/*
 *
 * TODO :
 * -
 *
 * Next version :
 * - Move each end points to resize components
 * - Add an options page (with increase or not the smoothing, Q-circles select, etc.)
 * - Display S21
 *
 *  Next next version :
 * - Er calculation for microstrip line & CPWG lines
*/

#include "mainwindow.h"

void MainWindow::simulationSlot(SimQuality _simQuality)
{
    qDebug() << "Simulation slot()";

    const QVector<complexFAndFreq> &startPointsNorm = m_smithChart->getStartPointsNorm();

    circuitSimulator simulator;
    if (m_forcedFastSimulation==true)
        simulator.startSimulation(startPointsNorm, *m_circuit, QUICK_SIM);
    else
        simulator.startSimulation(startPointsNorm, *m_circuit, _simQuality);

    m_smithChart->update();
    updateCircuitDisplayWidget();
    m_returnLossChart->update();
}


void MainWindow::targetPointCheckBoxClickedSlot(bool checked)
{
    m_smithChart->enableTargetPoints(checked);
    targetPointLineEditSlot();
}


void MainWindow::singleStartPointLineEditSlot()
{
    double tempValR, tempValI, tempValFreq;

    if ((checkRealPartImpValid(singleStartPointRealImpedanceLineEdit->displayText(), &tempValR))
            && (checkImagPartImpValid(singleStartPointImagImpedanceLineEdit->displayText(), &tempValI))
                && (checkFrequencyValid(singleFreqLineEdit->displayText(), &tempValFreq)))
    {
        singleStartPointRealImpedanceLineEdit->setStyleSheet("QLineEdit{background-color: white;}");
        singleStartPointImagImpedanceLineEdit->setStyleSheet("QLineEdit{background-color: white;}");
        singleFreqLineEdit->setStyleSheet("QLineEdit{background-color: white;}");
        complexFAndFreq complexFAndFreq1;
        complexFAndFreq1.point = complexF(tempValR,tempValI);
        complexFAndFreq1.freq = tempValFreq;
        m_startPointSelected = complexFAndFreq1;
        m_smithChart->setStartPoints(QVector<complexFAndFreq> (1, m_startPointSelected));
        m_circuit->setDefaultFrequency(tempValFreq);
        if (m_circuit->getSize()>0)
            simulationSlot(STANDARD_SIM);
        autoSave();
    }
    else
    {
        singleStartPointRealImpedanceLineEdit->setStyleSheet("QLineEdit{background-color: red;}");
        singleStartPointImagImpedanceLineEdit->setStyleSheet("QLineEdit{background-color: red;}");
        singleFreqLineEdit->setStyleSheet("QLineEdit{background-color: red;}");
        qDebug() << "startPointLineEditSlot() : conversion to double failed";
    }
}

void MainWindow::singleStartPointsLockButtonSlot()
{
    if (m_smithChart->getStartPointsLockState()==true) {
        m_smithChart->setStartPointsLockState(false);
        singleStartPointLockButton->setIcon(QIcon(":/myResources/Resources/unlocked.png"));
        multiStartPointsLockButton->setIcon(QIcon(":/myResources/Resources/unlocked.png"));
        singleStartPointRealImpedanceLineEdit->setEnabled(true);
        singleStartPointImagImpedanceLineEdit->setEnabled(true);
        singleFreqLineEdit->setEnabled(true);
    }
    else {
        m_smithChart->setStartPointsLockState(true);
        singleStartPointLockButton->setIcon(QIcon(":/myResources/Resources/locked.png"));
        multiStartPointsLockButton->setIcon(QIcon(":/myResources/Resources/locked.png"));
        singleStartPointRealImpedanceLineEdit->setEnabled(false);
        singleStartPointImagImpedanceLineEdit->setEnabled(false);
        singleFreqLineEdit->setEnabled(false);
    }
}

void MainWindow::multipleStartPointsLockButtonSlot()
{
    if (m_smithChart->getStartPointsLockState()==true) {
        m_smithChart->setStartPointsLockState(false);
        singleStartPointLockButton->setIcon(QIcon(":/myResources/Resources/unlocked.png"));
        multiStartPointsLockButton->setIcon(QIcon(":/myResources/Resources/unlocked.png"));
        multiStartPointsConfigButton->setEnabled(true);
    }
    else {
        m_smithChart->setStartPointsLockState(true);
        singleStartPointLockButton->setIcon(QIcon(":/myResources/Resources/locked.png"));
        multiStartPointsLockButton->setIcon(QIcon(":/myResources/Resources/locked.png"));
        multiStartPointsConfigButton->setEnabled(false);
    }
}

void MainWindow::multipleStartPointsConfigButtonSlot()
{
    complexFAndFreq currentValue;
    currentValue.point = complexF(singleStartPointRealImpedanceLineEdit->text().toDouble(), singleStartPointImagImpedanceLineEdit->text().toDouble());
    currentValue.freq = singleFreqLineEdit->text().toDouble();
    dialogPointsConfig myDialog (m_smithChart->getStartPointsNorm(), m_smithChart->getChartImpedance(), this);
    if (myDialog.exec() == QDialog::Accepted)
    {
        QVector<complexFAndFreq> returnVal = myDialog.getValues();
        if (returnVal.size() !=0 ) {
            //qDebug() << "Multi-points return 1 : " << complexNumberToQString(returnVal[0].point) << doubleToQString(returnVal[0].freq);
            //qDebug() << "Multi-points return 2 : " << complexNumberToQString(returnVal[1].point) << doubleToQString(returnVal[1].freq);
            singleStartPointRealImpedanceLineEdit->setText(doubleToQString(returnVal[0].point.real()));
            singleStartPointImagImpedanceLineEdit->setText(doubleToQString(returnVal[0].point.imag()));
            m_smithChart->setStartPoints(returnVal);
            if (m_circuit->getSize()>0)
                simulationSlot(STANDARD_SIM);
        }
    }
}


void MainWindow::startPointRBGroupSlot(int numButton)
{
    //qDebug() << "Id : " << numButton;
    if (numButton==0) // single point
    {
        multiStartPointsLabelConfig->setEnabled(false);
        multiStartPointsConfigButton->setEnabled(false);
        multiStartPointOnlyEndPointCheckBox->setEnabled(false);
        multiStartPointsLockButton->setEnabled(false);
        displayReturnLossCheckBox->setEnabled(false);
        singleStartPointImpedanceLabel->setEnabled(true);
        singleStartPointRealImpedanceLabel->setEnabled(true);
        singleStartPointImagImpedanceLabel->setEnabled(true);
        singleFreqLabel->setEnabled(true);
        singleFreqUnitLabel->setEnabled(true);
        singleStartPointLockButton->setEnabled(true);
        if (m_smithChart->getStartPointsLockState()==false) {
            singleStartPointRealImpedanceLineEdit->setEnabled(true);
            singleStartPointImagImpedanceLineEdit->setEnabled(true);
            singleFreqLineEdit->setEnabled(true);
        }
        singleStartPointLineEditSlot();
        m_returnLossChart->hide();
    }
    else
    {
        multiStartPointsLabelConfig->setEnabled(true);
        if (m_smithChart->getStartPointsLockState()==true)
            multiStartPointsConfigButton->setEnabled(false);
        else
            multiStartPointsConfigButton->setEnabled(true);
        multiStartPointOnlyEndPointCheckBox->setEnabled(true);
        multiStartPointsLockButton->setEnabled(true);
        displayReturnLossCheckBox->setEnabled(true);
        singleStartPointImpedanceLabel->setEnabled(false);
        singleStartPointRealImpedanceLabel->setEnabled(false);
        singleStartPointRealImpedanceLineEdit->setEnabled(false);
        singleStartPointImagImpedanceLabel->setEnabled(false);
        singleStartPointImagImpedanceLineEdit->setEnabled(false);
        singleFreqLabel->setEnabled(false);
        singleFreqLineEdit->setEnabled(false);
        singleFreqUnitLabel->setEnabled(false);
        singleStartPointLockButton->setEnabled(false);
    }
}

void MainWindow::displayReturnLossCheckBoxClickedSlot(bool checked)
{
    if (checked == true)
    {
        m_returnLossChart->show();
        this->activateWindow();
    }
    else
        m_returnLossChart->hide();
}

void MainWindow::displayReturnLossCheckBoxClosedSlot()
{
    displayReturnLossCheckBox->setChecked(false);
}


void MainWindow::startPointsUpdateFromChartSlot()
{
    QVector<complexFAndFreq> normalizedPoints = m_smithChart->getStartPointsNorm();
    if (normalizedPoints.size()>=1) // check at least one point
    {
        complexF denormPoint = normalizedPoints[0].point*m_smithChart->getChartImpedance();
        qDebug() << "Start point update from Chart : " << complexNumberToQString(denormPoint);

        singleStartPointRealImpedanceLineEdit->setText(doubleToQString(denormPoint.real()));
        singleStartPointImagImpedanceLineEdit->setText(doubleToQString(denormPoint.imag()));
    }
}
void MainWindow::targetPointUpdateFromChartSlot()
{
    complexF normalizedNewPoint = m_smithChart->getTargetPoint();
    complexF denormPoint = normalizedNewPoint*m_smithChart->getChartImpedance();
    qDebug() << "Target point update from Chart : " << complexNumberToQString(denormPoint);

    targetPointRealImpedanceLineEdit->setText(doubleToQString(denormPoint.real()));
    targetPointImagImpedanceLineEdit->setText(doubleToQString(denormPoint.imag()));

    updateCircuitDisplayWidget();
}

void MainWindow::targetPointLineEditSlot()
{
    double tempValR, tempValI;

    tempValR = targetPointRealImpedanceLineEdit->displayText().toDouble();
    tempValI = targetPointImagImpedanceLineEdit->displayText().toDouble();

    if (tempValR > 0.0)
    {
        targetPointRealImpedanceLineEdit->setStyleSheet("QLineEdit{background-color: white;}");
        targetPointImagImpedanceLineEdit->setStyleSheet("QLineEdit{background-color: white;}");
        m_smithChart->setTargetPoint(complexF(tempValR,tempValI));
        //autoSave();
    }
    else
    {
        targetPointRealImpedanceLineEdit->setStyleSheet("QLineEdit{background-color: red;}");
        targetPointImagImpedanceLineEdit->setStyleSheet("QLineEdit{background-color: red;}");
    }
}

void MainWindow::targetPointLockButtonSlot()
{
    if (m_smithChart->getTargetPointLockState()==true) {
        m_smithChart->setTargetPointLockState(false);
        targetPointLockButton->setIcon(QIcon(":/myResources/Resources/unlocked.png"));
        targetPointRealImpedanceLineEdit->setEnabled(true);
        targetPointImagImpedanceLineEdit->setEnabled(true);
    }
    else {
        m_smithChart->setTargetPointLockState(true);
        targetPointLockButton->setIcon(QIcon(":/myResources/Resources/locked.png"));
        targetPointRealImpedanceLineEdit->setEnabled(false);
        targetPointImagImpedanceLineEdit->setEnabled(false);
    }
}


void MainWindow::changeChartImpedanceCheckBoxSlot(bool checked)
{
    if (checked==false)
    {
        chartImpedanceLineEdit->setText("50.0");
        m_smithChart->setChartImpedance(50.0);
    }

    changeChartImpedanceSlot();
    m_smithChart->update();
}

void MainWindow::changeChartImpedanceSlot()
{
    qDebug() << "changeChartImpedanceSlot()";
    double tempVal;
    bool convSuccess=false;

    if (chartImpedanceLineEdit->displayText()=="50")
        chartImpedanceLineEdit->setText("50.0");

    tempVal = chartImpedanceLineEdit->displayText().toDouble(&convSuccess);

    if ((tempVal > 0.0) && (convSuccess==true))
    {
        chartImpedanceLineEdit->setStyleSheet("QLineEdit{background-color: white;}");
        m_chartImpedanceSelected = tempVal;
        m_smithChart->setChartImpedance(m_chartImpedanceSelected);
        singleStartPointLineEditSlot(); // replace the start point
        targetPointLineEditSlot(); // replace the target point
    }
    else
    {
        chartImpedanceLineEdit->setStyleSheet("QLineEdit{background-color: red;}");
    }
}


/* Called when "Clear all" button is clicked */
void MainWindow::clearAllSlot()
{
    qDebug() << "clearAllSlot()";
    m_circuit->clearAll();
    m_smithChart->clearStartImpedancePoints();
    m_smithChart->clearTargetImpedancePoint();
    m_smithChart->setChartImpedance(50.0);
    chartImpedanceLineEdit->setText("50.0");
}

/* Called when a component is modified or deleted */
void MainWindow::updateCircuitDisplayWidget()
{
    const int _HEIGHT_ITEM=52;
    const int _WIDTH_ITEM=350;
    const int _HEIGHT_PIX_ITEM=48;
    circuitListView->clear();

    // Painter initialization
    QPixmap pixmap(_WIDTH_ITEM,_HEIGHT_ITEM);
    pixmap.fill(Qt::transparent);
    QPainter painter(&pixmap);
    painter.setFont(QFont("monospace", 10, QFont::Normal));
    //painter.setPen(QPen(QColor("black"), 2.0, Qt::SolidLine, Qt::FlatCap));

    // Start point
    complexF reflectionCoeff = ZloadToCoeffReflect(m_smithChart->getFirstStartPointDenorm(), m_smithChart->getChartImpedance());
    complexF reflectionCoeffPolar = cartToPolar(reflectionCoeff);
    double returnLoss = 20.0*log10(reflectionCoeffPolar.real());

    QListWidgetItem *itemStart = new QListWidgetItem();
    itemStart->setSizeHint(QSize(itemStart->sizeHint().width(), _HEIGHT_ITEM));
    QLabel *labelItemStart = new QLabel();
    circuitListView->addItem(itemStart);
    circuitListView->setItemWidget(itemStart, labelItemStart);
    QString textItem;
    if (m_smithChart->getStartPointsNorm().size()==1) { // only one start point
        textItem = QString(tr("-> Start point : ")) + complexNumberToQString(m_smithChart->getFirstStartPointDenorm());
        painter.drawText(QPointF(_WIDTH_ITEM*0.25,_HEIGHT_ITEM*0.43), textItem);
    } else { // multiple start points
        textItem = QString(tr("-> (First) start point : ")) + complexNumberToQString(m_smithChart->getFirstStartPointDenorm());
        painter.drawText(QPointF(_WIDTH_ITEM*0.25,_HEIGHT_ITEM*0.43), textItem);
    }
    QString textReturnLoss = QString(tr("Return loss (dB) : ")) + doubleToQString(returnLoss);
    painter.drawText(QPointF(_WIDTH_ITEM*0.25,_HEIGHT_ITEM*0.72), textReturnLoss);
    QPixmap pixDotAndPoint = m_circuit->getPixmapDotAndPoint(_HEIGHT_PIX_ITEM,_HEIGHT_PIX_ITEM);
    painter.drawPixmap(5,2, pixDotAndPoint);
    labelItemStart->setPixmap(pixmap);
    labelItemStart->setToolTip(textItem + QString("\n") + textReturnLoss);

    // List of circuit components
    for (int i=0 ; i<m_circuit->getSize() ; i++)
    {
        if (m_circuit->getComponentPreviewState(i)!=true) // added on list only if not a previewed component
        {
            ComponentType tmpCmpType = m_circuit->getComponentType(i);

            QListWidgetItem *itemComp = new QListWidgetItem();
            itemComp->setSizeHint(QSize(itemComp->sizeHint().width(), _HEIGHT_ITEM));
            QLabel *labelItemComp = new QLabel();
            circuitListView->addItem(itemComp);
            circuitListView->setItemWidget(itemComp, labelItemComp);
            QPixmap compPixmapTmp = m_circuit->getPixmapComponent(tmpCmpType, _HEIGHT_PIX_ITEM, _HEIGHT_PIX_ITEM);
            pixmap.fill(Qt::transparent);
            painter.drawPixmap(5,2, compPixmapTmp);

            if ((tmpCmpType != LINE_S)&&(tmpCmpType != LINE_OPEN)&&(tmpCmpType != LINE_SHORT))
            {
                double compValTmp = m_circuit->getComponentParameterDenorm(i, 0, m_smithChart->getChartImpedance());

                painter.drawText(QPointF(_WIDTH_ITEM*0.25,_HEIGHT_ITEM*0.43), QString::number(i+1) + QString(") ") +
                                                                            m_circuit->getComponentName(i));
                painter.drawText(QPointF(_WIDTH_ITEM*0.31,_HEIGHT_ITEM*0.72), doubleToQString(compValTmp) + \
                                                                             m_circuit->getComponentStrUnit(i));
            }
            else
            {
                double compLengthTmp = m_circuit->getComponentParameterDenorm(i, 0, m_smithChart->getChartImpedance()); // mechanical length in mm
                double compImpedTmp = m_circuit->getComponentParameterDenorm(i, 1, m_smithChart->getChartImpedance()); // impedance
                double compPermTmp = m_circuit->getComponentParameter(i, 2); // er
                double lambda = 3.0e2 / (m_smithChart->getStartPointsNorm().first().freq * sqrt(compPermTmp)); // 1*λ in mm (wavelength)

                painter.drawText(QPointF(_WIDTH_ITEM*0.25,_HEIGHT_ITEM*0.43), QString::number(i+1) + QString(") ") \
                                 + m_circuit->getComponentName(i) + " " \
                                 + QString::number(compImpedTmp)  + "Ω");
                painter.drawText(QPointF(_WIDTH_ITEM*0.31,_HEIGHT_ITEM*0.72), QString("")
                                 + "L=" + QString::number(compLengthTmp) + m_circuit->getComponentStrUnit(i) + " (" + QString::number(compLengthTmp/lambda,'f', 3) + "λ)"
                                 + "  Er=" + QString::number(compPermTmp) + " ");
            }

            labelItemComp->setPixmap(pixmap);
        }
    }


    // End point
    if (m_circuit->getSizeWithoutPreviewed()>0)
    {
        if (m_circuit->getComponentDataPoints(m_circuit->getSizeWithoutPreviewed()-1).size()>0)
        {
            complexF lastDataPoint = m_circuit->getComponentDataPoints(m_circuit->getSizeWithoutPreviewed()-1)[0].last();
            reflectionCoeff = ZloadToCoeffReflect(lastDataPoint*m_smithChart->getChartImpedance(), m_smithChart->getChartImpedance());
            reflectionCoeffPolar = cartToPolar(reflectionCoeff);
            returnLoss = 20.0*log10(reflectionCoeffPolar.real());

            QListWidgetItem *itemEnd = new QListWidgetItem();
            itemEnd->setSizeHint(QSize(itemEnd->sizeHint().width(), _HEIGHT_ITEM));
            QLabel *labelItemEnd = new QLabel();
            circuitListView->addItem(itemEnd);
            circuitListView->setItemWidget(itemEnd, labelItemEnd);
            QTransform rota180;
            rota180 = rota180.rotate(180);
            QPixmap pixDotAndPoint = m_circuit->getPixmapDotAndPoint(_HEIGHT_PIX_ITEM,_HEIGHT_PIX_ITEM);
            pixmap.fill(Qt::transparent);
            painter.drawPixmap(5,2,pixDotAndPoint.transformed(rota180));
            QString textItem;
            if (m_smithChart->getStartPointsNorm().size()==1) { // only one end point
                textItem = QString(tr("-> End point : ")) + complexNumberToQString(lastDataPoint*m_smithChart->getChartImpedance());
                painter.drawText(QPointF(_WIDTH_ITEM*0.25,_HEIGHT_ITEM*0.43), textItem);
            } else { // multiple end points
                textItem = QString(tr("-> (First) end point : ")) + complexNumberToQString(lastDataPoint*m_smithChart->getChartImpedance());
                painter.drawText(QPointF(_WIDTH_ITEM*0.25,_HEIGHT_ITEM*0.43), textItem);
            }
            QString textReturnLoss = QString(tr("Return loss (dB) : ")) + doubleToQString(returnLoss);
            painter.drawText(QPointF(_WIDTH_ITEM*0.25,_HEIGHT_ITEM*0.72), QString(tr("Return loss (dB) : ")) + doubleToQString(returnLoss) );
            labelItemEnd->setPixmap(pixmap);
            labelItemEnd->setToolTip(textItem + QString("\n") + textReturnLoss);
        }
    }
}



void MainWindow::updateStatusBar()
{
    // Impedance Z calculation
    double chartImpedance = m_smithChart->getChartImpedance();
    complexF impedanceMouse = m_smithChart->getLastZPosMouse() * chartImpedance;

    complexF admittanceMouse = (complexF(1.0, 0.0)/impedanceMouse)*1000.0;

    complexF reflectionCoeff = ZloadToCoeffReflect(impedanceMouse, chartImpedance);
    complexF reflectionCoeffPolar = cartToPolar(reflectionCoeff);
    double returnLoss = 20.0*log10(reflectionCoeffPolar.real());

    QString myStr = "Z = (" + complexNumberToQString(impedanceMouse) + ")Ω" +
            "        Y = (" + complexNumberToQString(admittanceMouse) + ")mS" +
            "        Γ = " + doubleToQString(reflectionCoeffPolar.real()) + "/" + doubleToQString(reflectionCoeffPolar.imag()) + "°" +
            "        Return loss = " + doubleToQString(returnLoss) + " dB" +
            "        Q = " + doubleToQString(abs(impedanceMouse.imag()/impedanceMouse.real())) +
            "        VSWR = " + doubleToQString(abs((1.0+abs(reflectionCoeff))/(1.0-abs(reflectionCoeff)))) + " :1";
    statusBar()->showMessage(myStr);
}


void MainWindow::doubleClickInCompListSlot(QModelIndex index)
{
    int indexClicked = index.row();
    if ((indexClicked >= 1) && (indexClicked <= circuitListView->count()-2))
        m_smithChart->componentDoubleClick(indexClicked-1);
}

void MainWindow::circuitListViewContextMenuRequested(const QPoint &pos)
{
    // CAPA_S, SELF_S, RESIST_S, LINE_S, CAPA_P, SELF_P, RESIST_P, LINE_OPEN, LINE_SHORT, NOT_DEF };
    QAction actionRemoveComp(this);
    QAction actionInsertCAPA_S(this), actionInsertSELF_S(this), actionInsertRESIST_S(this), actionInsertLINE_S(this), actionInsertCAPA_P(this), actionInsertSELF_P(this), actionInsertRESIST_P(this), actionInsertLINE_OPEN(this), actionInsertLINE_SHORT(this);
    int numItem;
    if (circuitListView->currentIndex().row()==0) // if start point selected, action on item after (like insert capa after)
    {
        numItem = 0;
        actionRemoveComp.setText(tr("Remove this component"));
        actionInsertCAPA_S.setText(tr("Insert a serial capacitor (after)"));
        actionInsertSELF_S.setText(tr("Insert a serial inductor (after)"));
        actionInsertRESIST_S.setText(tr("Insert a serial resistor (after)"));
        actionInsertLINE_S.setText(tr("Insert a serial line (after)"));
        actionInsertCAPA_P.setText(tr("Insert a parallel capacitor (after)"));
        actionInsertSELF_P.setText(tr("Insert a parallel self (after)"));
        actionInsertRESIST_P.setText(tr("Insert a parallel resistor (after)"));
        actionInsertLINE_OPEN.setText(tr("Insert a parallel line open (after)"));
        actionInsertLINE_SHORT.setText(tr("Insert a parallel line shorted (after)"));
    }
    else // other item, action on item before (like insert capa before)
    {
        numItem= circuitListView->currentIndex().row()-1;
        actionRemoveComp.setText(tr("Remove this component"));
        actionInsertCAPA_S.setText(tr("Insert a serial capacitor (before)"));
        actionInsertSELF_S.setText(tr("Insert a serial inductor (before)"));
        actionInsertRESIST_S.setText(tr("Insert a serial resistor (before)"));
        actionInsertLINE_S.setText(tr("Insert a serial line (before)"));
        actionInsertCAPA_P.setText(tr("Insert a parallel capacitor (before)"));
        actionInsertSELF_P.setText(tr("Insert a parallel self (before)"));
        actionInsertRESIST_P.setText(tr("Insert a parallel resistor (before)"));
        actionInsertLINE_OPEN.setText(tr("Insert a parallel line open (before)"));
        actionInsertLINE_SHORT.setText(tr("Insert a parallel line shorted (before)"));
    }

    QMenu submenu;
    if ((circuitListView->currentIndex().row()>=1) && (circuitListView->currentIndex().row()<circuitListView->count()-1)) {
        submenu.addAction(&actionRemoveComp);
    }
    submenu.addAction(&actionInsertCAPA_S);
    submenu.addAction(&actionInsertSELF_S);
    submenu.addAction(&actionInsertRESIST_S);
    submenu.addAction(&actionInsertLINE_S);
    submenu.addAction(&actionInsertCAPA_P);
    submenu.addAction(&actionInsertSELF_P);
    submenu.addAction(&actionInsertRESIST_P);
    submenu.addAction(&actionInsertLINE_OPEN);
    submenu.addAction(&actionInsertLINE_SHORT);

    QPoint posMenu = circuitListView->mapToGlobal(pos);
    QAction* rightClickItem = submenu.exec(posMenu);

    if (rightClickItem) {
        if (rightClickItem->text() == actionRemoveComp.text() ) {
                m_circuit->removeComponent(numItem);
        }
        else if (rightClickItem->text() == actionInsertCAPA_S.text()) {
                m_circuit->insertCAPA_S(numItem);
        }
        else if (rightClickItem->text() == actionInsertSELF_S.text()) {
            m_circuit->insertSELF_S(numItem);
        }
        else if (rightClickItem->text() == actionInsertRESIST_S.text()) {
            m_circuit->insertRESIST_S(numItem);
        }
        else if (rightClickItem->text() == actionInsertLINE_S.text()) {
            m_circuit->insertLINE_S(numItem);
        }
        else if (rightClickItem->text() == actionInsertCAPA_P.text()) {
            m_circuit->insertCAPA_P(numItem);
        }
        else if (rightClickItem->text() == actionInsertSELF_P.text()) {
            m_circuit->insertSELF_P(numItem);
        }
        else if (rightClickItem->text() == actionInsertRESIST_P.text()) {
            m_circuit->insertRESIST_P(numItem);
        }
        else if (rightClickItem->text() == actionInsertLINE_OPEN.text()) {
            m_circuit->insertLINE_OPEN(numItem);
        }
        else if (rightClickItem->text() == actionInsertLINE_SHORT.text()) {
            m_circuit->insertLINE_SHORT(numItem);
        }
        updateCircuitDisplayWidget();
        simulationSlot(STANDARD_SIM);
    }
}


void MainWindow::saveFileActionSlot()
{
    if (m_saveFileName->isEmpty())
        saveFileAsActionSlot();
    else
        saveFile(*m_saveFileName);
}

void MainWindow::saveFileAsActionSlot()
{
    *m_saveFileName = QFileDialog::getSaveFileName(this, tr("Save circuit"),
                               QString(QDir::homePath() + QString("/Desktop/")),
                               tr("QSmithTool file (*.qst)"));
    saveFile(*m_saveFileName);
}

void MainWindow::saveFile(QString _fileName)
{
    QFile file(_fileName);

    // Write the conf file for selected colors
    if (file.open(QIODevice::WriteOnly)) {
        QTextStream stream(&file);
        stream << "#" << _SOFT_NAME << " circuit file" << endl;
        stream << "[Chart impedance]" << endl;
        stream << QString::number(m_smithChart->getChartImpedance()) << endl;
        stream << "[Start points]" << endl;
        const QVector<complexFAndFreq> &startPointsNorm = m_smithChart->getStartPointsNorm();
        for (int i=0 ; i<startPointsNorm.size() ; i++) {
            stream << QString::number(startPointsNorm[i].freq) \
                   << QString(' ') \
                   << QString::number(startPointsNorm[i].point.real()) \
                   << QString(' ') \
                   << QString::number(startPointsNorm[i].point.imag()) \
                   << QString(' ') \
                   << endl;
        }
        stream << "[Target point]" << endl;
        stream << QString::number(m_smithChart->getTargetPoint().real()) \
               << QString(' ') \
               << QString::number(m_smithChart->getTargetPoint().imag()) \
               << endl;
        stream << "[Components list]" << endl;
        for (int i=0 ; i<m_circuit->getSize() ; i++) {
            stream << QString::number(m_circuit->getComponentType(i)) \
                   << QString(' ');
            for (int j=0 ; j<m_circuit->getComponentParameters(i).size() ; j++) {
                stream   << QString::number(m_circuit->getComponentParameter(i, j)) \
                         << QString(' ');
            }
            stream  << endl;
        }
        file.close();
    }
}

void MainWindow::openFileActionSlot()
{
    // Ask user for file path
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open circuit file"),
                                                    QString(QDir::homePath() + QString("/Desktop/")),
                                                    tr("QSmithTool file (*.qst)"));


    if (fileName.isEmpty()) // user cancel
        return;
    else {
        openFile(fileName);
        clearAllAutoSaveFiles();
        autoSave();
    }

}


void MainWindow::openFile(QString _fileName)
{
    enum CIRCUIT_FILE_SECTION { NO_SECTION, CHART_IMP, START_POINTS, TARGET_POINT, COMPONENTS_LIST };
    CIRCUIT_FILE_SECTION fileSection = NO_SECTION;

    QVector<complexFAndFreq> startPointsList;

    QFile file(_fileName);

    if (file.exists())
    {
        QString line;
        //QStringList splittedLine;
        if (file.open(QIODevice::ReadOnly)) {
            QTextStream stream(&file);
            line = stream.readLine(); // jump to next line
            if (line.contains("#QSmithTool"))
            {
                qDebug() << "Opening circuit file : " << _fileName;
                clearAllSlot();
                while (!stream.atEnd())
                {
                    line = stream.readLine(); // jump to next line
                    qDebug() << "Line read : " << line;

                    if (line.at(0)=='[') // section title line
                    {
                        if (line.contains("[Chart impedance]"))
                            fileSection = CHART_IMP;
                        else if (line.contains("[Start points]")) {
                            fileSection = START_POINTS;
                        }
                        else if (line.contains("[Target point]")) {
                            if (fileSection == START_POINTS) {
                                m_smithChart->setStartPointsNormalized(startPointsList);
                                startPointsUpdateFromChartSlot();
                            }
                            fileSection = TARGET_POINT;
                        }
                        else if (line.contains("[Components list]"))
                            fileSection = COMPONENTS_LIST;
                    }
                    else // data line
                    {
                        if (fileSection == CHART_IMP) {
                            chartImpedanceGroupBox->setChecked((line != "50"));
                            chartImpedanceLineEdit->setText(line);
                            m_smithChart->setChartImpedance(line.toDouble());
                        }
                        else if (fileSection == START_POINTS) {
                            complexFAndFreq pt;
                            pt.freq = line.split(' ').at(0).toDouble();
                            pt.point.real() = line.split(' ').at(1).toDouble();
                            pt.point.imag() = line.split(' ').at(2).toDouble();
                            startPointsList.append(pt);
                            qDebug() << startPointsList.size();
                            if (startPointsList.size()>1) { // multi points
                                multiStartPointsRadioButton->setChecked(true);
                                startPointRBGroupSlot(startPointRadioButtonGroup->id(multiStartPointsRadioButton));
                            }

                        }
                        else if (fileSection == TARGET_POINT) {
                            complexF pt;
                            pt.real() = line.split(' ').at(0).toDouble();
                            pt.imag() = line.split(' ').at(1).toDouble();
                            m_smithChart->setTargetPointNormalized(pt);
                            targetPointUpdateFromChartSlot();

                        }
                        else if (fileSection == COMPONENTS_LIST) {
                            CompParametersList paramList;
                            for (int i=1; i<line.split(' ').size()-1 ; i++) {
                            paramList.append(line.split(' ').at(i).toDouble()); // 24mm mechanical length by default at 1GHz
                            }
                            m_circuit->addComponent((ComponentType)line.split(' ').at(0).toInt(), paramList);
                        }
                    }
                }
            } // end if (line.contains("#QSmithTool"))
            file.close();

            m_smithChart->setFocus(Qt::TabFocusReason);
            m_smithChart->startSim(STANDARD_SIM);
            m_smithChart->update();
            return;
        }
    }
    QMessageBox *msgBox = new QMessageBox(QMessageBox::Warning, "Error", "Invalid QSmithTool format");
    msgBox->show();
}


void MainWindow::autoSave()
{
    if (m_autoSaveEnable==true) {
        QFile lastAutoSaveFile;
        QDir folder("./");

        folder.mkdir("backup");

        lastAutoSaveFile.setFileName(QString("./backup/backup#")+QString::number(m_numLastAutosave)+QString(".qst"));
        m_numLastAutosave++;

        saveFile(lastAutoSaveFile.fileName());
        qDebug() << "Autosave() : " << lastAutoSaveFile.fileName();
    }
}
void MainWindow::loadPreviousAutoSave()
{
    QFile previousAutoSaveFile, lastAutoSaveFile;

    previousAutoSaveFile.setFileName(QString("./backup/backup#")+QString::number(m_numLastAutosave-2)+QString(".qst"));
    lastAutoSaveFile.setFileName(QString("./backup/backup#")+QString::number(m_numLastAutosave-1)+QString(".qst"));

    if (previousAutoSaveFile.exists()) {
        qDebug() << "Loading autosave :" << previousAutoSaveFile.fileName();
        openFile(previousAutoSaveFile.fileName());
        if (lastAutoSaveFile.exists())
            lastAutoSaveFile.remove();
        m_numLastAutosave--;
    }
    else
        qDebug() << "No autosave to load";
}
void MainWindow::clearAllAutoSaveFiles()
{
    QFile autoSaveFile;
    bool lastWasExisting=false;
    int i=0;
    do {
        lastWasExisting=false;
        autoSaveFile.setFileName(QString("./backup/backup#")+QString::number(i)+QString(".qst"));
        if (autoSaveFile.exists()) {
            i++;
            lastWasExisting=true;
            autoSaveFile.remove();
        }
    } while (lastWasExisting==true);
    qDebug() << "Number of autosave files removed : " << i;
    m_numLastAutosave=0;
}

void MainWindow::forceFastSimActionSlot(bool _checked)
{
    if (_checked==true) {
        qDebug() << "Fast simulation parameter forced";
        m_forcedFastSimulation=true;
        startSmoothRenderAction->setEnabled(false);
    }
    else {
        qDebug() << "Normal simulation parameter";
        m_forcedFastSimulation=false;
        startSmoothRenderAction->setEnabled(true);
    }
    simulationSlot();
    m_smithChart->update();
}
void MainWindow::startSmoothRenderActionSlot()
{
    qDebug() << "Smooth render()";
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    simulationSlot(HQ_SIM);
    m_smithChart->update();
    QApplication::restoreOverrideCursor();
}


void MainWindow::displayQCirclesActionSlot(bool _checked)
{
    qDebug() << "Display QCircle :" << _checked;
    m_smithChart->enableQCirclesDrawing(_checked);
}


void MainWindow::languageSelectedSlot()
{
    QMessageBox msgBox;
    msgBox.setIcon(QMessageBox::Information);
    msgBox.setWindowTitle(tr("Modification completed"));

    QString filename = "./config.txt";
    QFile file(filename);

    QString langSelected = languageGroup->checkedAction()->text();
    if (langSelected==FRAction->text())
    {
        // Write here the conf file for selected language
        if (file.open(QIODevice::ReadWrite)) {
            QTextStream stream(&file);
            stream << "[Language]" << endl;
            stream << "FRA" << endl;
            file.close();
            qDebug() << "Written FRA";
        }
        //
        qDebug() << "Written FRA";
        QString textMsgbox = QString(tr("Language modified to \'")) + langSelected + QString(tr("\' , modification will take effect at the next startup."));
        msgBox.setText(textMsgbox);
        msgBox.exec();
        qDebug() << QString("Language selected : ") + langSelected;
    }
    else if (langSelected==ENGAction->text())
    {
        // Write here the conf file for selected language
        if (file.open(QIODevice::ReadWrite)) {
            QTextStream stream(&file);
            stream << "[Language]" << endl;
            stream << "ENG" << endl;
            file.close();
            qDebug() << "Written ENG";
        }
        //
        QString textMsgbox = QString(tr("Language modified to \'")) + langSelected + QString(tr("\' , modification will take effect at the next startup."));
        msgBox.setText(textMsgbox);
        msgBox.exec();
        qDebug() << QString("Language selected : ") + langSelected;
    }
    else
        qDebug() << "No language selected"; //impossible
}


void MainWindow::shortcutHelpSlot()
{
    QString textShortcuts = tr("<html><body>"
                "<strong style=\"color: blue;\">Mouse shortcuts :</strong><br>" \
                          "<table>" \
                              "<tr>" \
                                "<td width=\"180\"><strong>Left click on '+' icon</strong></td>"
                                "<td>Add a new component</td>" \
                              "</tr>" \
                              "<tr>" \
                                "<td><strong>Right click </strong></td>"
                                "<td>Remove the last added component</td>" \
                              "</tr>" \
                              "<tr>" \
                                "<td><strong>Scroll on a component<br>with 'Ctrl' key</strong></td>"
                                "<td>Increase or decrease value/length</td>" \
                              "</tr>" \
                              "<tr>" \
                                "<td><strong>Scroll on the chart </strong></td>"
                                "<td>Zoom+/zoom-</td>" \
                              "</tr>" \
                              "<tr>" \
                                "<td><strong>Double click on a component</strong></td>"
                                "<td>Modify a parameter/length<br></td>" \
                              "</tr>" \
                          "</table>"
                          "<strong style=\"color:blue;\">Keyboard shortcuts :</strong><br>" \
                            "<table>" \
                                "<tr>" \
                                    "<td width=\"180\"><strong>Ctrl+Z</strong></td>"
                                    "<td>Undo</td>" \
                                "</tr>" \
                                "<tr>" \
                                    "<td><strong>Key 'C' & Shift+'C'</strong></td>"
                                    "<td>Add a parallel cap & serie cap</td>" \
                                "</tr>" \
                               "<tr>" \
                                   "<td><strong>Key 'L' & Shift+'L'</strong></td>"
                                   "<td>Add a transmission line & open stub</td>" \
                               "</tr>" \
                               "<tr>" \
                                   "<td><strong>Key 'I' & Shift+I'</strong></td>"
                                   "<td>Add a parallel inductor & serie inductor</td>" \
                               "</tr>" \
                               "<tr>" \
                                   "<td><strong>Key 'R' & Shift+R'</strong></td>"
                                   "<td>Add a parallel resistor & serie resistor</td>" \
                               "</tr>" \
                                "<tr>" \
                                    "<td><strong>Key 'H'</strong></td>"
                                    "<td>Hide the supp component icon ('+')</td>" \
                                "</tr>" \
                            "</table>"
                          "</body></html>");

    //QMessageBox::about(this, _SOFT_NAME, textShortcuts);

    QDialog *myDialog = new QDialog();
    myDialog->setStyleSheet("QDialog { background-color : white;}");
    myDialog->setWindowTitle("Shortcuts list");
    QLabel *textDialog = new QLabel(textShortcuts);
    textDialog->setStyleSheet("QLabel { background-color : white;}");
    QLabel *labelPixQSmithTool = new QLabel();
    labelPixQSmithTool->setStyleSheet("QLabel { background-color : white;}");
    QPixmap *QSmithToolPixmap= new QPixmap(QString(":/myResources/Resources/appIcoPNG.png")) ;
    labelPixQSmithTool->setPixmap(QSmithToolPixmap->scaled(60,60));
    QVBoxLayout *layoutPixmap = new QVBoxLayout();
    layoutPixmap->addWidget(labelPixQSmithTool);
    layoutPixmap->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Minimum, QSizePolicy::Expanding));

    QPushButton *okButton = new QPushButton("Ok");
    connect(okButton, SIGNAL(clicked(bool)), myDialog, SLOT(accept()));
    QHBoxLayout *Hlayout1 = new QHBoxLayout();
    Hlayout1->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Expanding, QSizePolicy::Minimum));
    Hlayout1->addWidget(okButton);

    QHBoxLayout *Hlayout2 = new QHBoxLayout();
    Hlayout2->addLayout(layoutPixmap);
    Hlayout2->addWidget(textDialog);
    QVBoxLayout *Vlayout2 = new QVBoxLayout();
    Vlayout2->addLayout(Hlayout2);
    Vlayout2->addLayout(Hlayout1);
    myDialog->setLayout(Vlayout2);
    myDialog->exec();
}


void MainWindow::aboutSlot()
{
    QMessageBox::about(this, _SOFT_NAME, tr("QSmithTool is a software developped by Sebastien L.\nfor LPA Concepts"));
}


void MainWindow::closeEvent(QCloseEvent *)
{
    m_returnLossChart->close();
}

void MainWindow::keyPressEvent( QKeyEvent *event )
{
    // Propagate the keyPressEvent to smithChart
    m_smithChart->keyPressEvent(event);
}

void MainWindow::wheelEvent ( QWheelEvent *)
{
}

MainWindow::MainWindow()
{
    qDebug() << "Reading configuration...";

    /// Read config file and install translator
    QString filename = "./config.txt";
    QFile file(filename);
    QString line;
    if (file.open(QIODevice::ReadOnly)) {
        QTextStream stream(&file);
        line = stream.readLine();
        line = stream.readLine();
        file.close();
    }

    translator = new QTranslator;
    if (line==QString("FRA"))
    {
        translator->load(":/myResources/Resources/Translations/QSmithTool_fr.qm");
        qApp->installTranslator(translator);
        languageSelected=1;
        qDebug() << "Language FRA selected";
    }
    else
    {
        translator->load(":/myResources/Resources/Translations/QSmithTool_en.qm");
        qApp->installTranslator(translator);
        languageSelected=0;
        qDebug() << "Language ENG selected, line:" << line ;
    }

    qDebug() << "Reading configuration... OK";
    qDebug() << "Deleting last session autosave files...";

    clearAllAutoSaveFiles();
    m_autoSaveEnable=false;

    qDebug() << "Deleting last session autosave files... OK";
    qDebug() << "Initializing Circuit...";

    m_circuit = new Circuit(1.0);

    qDebug() << "Initializing Circuit... OK";
    qDebug() << "Initializing Smith Chart...";

    m_chartImpedanceSelected = 50.0;
    m_smithChart = new SmithChart(m_chartImpedanceSelected, *m_circuit);
    m_smithChart->setMouseTracking(true);
    m_smithChart->setFocusPolicy(Qt::WheelFocus);

    qDebug() << "Initializing Smith Chart... OK";
    qDebug() << "Initializing Return Loss Chart...";

    m_returnLossChart = new ReturnLossChart(this, *m_circuit, m_smithChart->getStartPointsNorm());
    m_returnLossChart->hide();

    qDebug() << "Initializing Return Loss Chart... OK";
    qDebug() << "Drawing windows...";

    startPointRadioButtonGroup = new QButtonGroup(this);
    //freqLayoutH2->addItem(new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum));

    // First page of the dataPage toolBox
    /* Groupbox for single start point */
    singlePointGroupBox = new QGroupBox ("                                     ", this) ;
    singlePointRadioButton = new QRadioButton(tr("Single start point"), singlePointGroupBox);
    singlePointRadioButton->move(14, -8);
    singlePointRadioButton->setChecked(true);
    singleStartPointImpedanceLabel = new QLabel(tr("Point impedance :"), this);
    singleStartPointRealImpedanceLabel = new QLabel("r", this);
    singleStartPointRealImpedanceLineEdit = new QLineEdit("15.0", this);
    singleStartPointImagImpedanceLabel = new QLabel("i");
    singleStartPointImagImpedanceLineEdit = new QLineEdit("-10.0", this);
    singleStartPointLockButton = new QPushButton();
    singleStartPointLockButton->setMaximumWidth(20);
    singleStartPointLockButton->setIcon(QIcon(":/myResources/Resources/unlocked.png"));
    singleStartPointLayoutH1 = new QHBoxLayout();
    singleStartPointLayoutH1->addWidget(singleStartPointImpedanceLabel);
    singleStartPointLayoutH1->addWidget(singleStartPointRealImpedanceLabel);
    singleStartPointLayoutH1->addWidget(singleStartPointRealImpedanceLineEdit);
    singleStartPointLayoutH1->addWidget(singleStartPointImagImpedanceLabel);
    singleStartPointLayoutH1->addWidget(singleStartPointImagImpedanceLineEdit);
    singleStartPointLayoutH1->addWidget(singleStartPointLockButton);
    singleFreqLabel = new QLabel(tr("Frequency :"), this);
    singleFreqLineEdit = new QLineEdit(doubleToQString(1.0), this);
    singleFreqUnitLabel = new QLabel("GHz", this);
    singleStartPointLayoutH2 = new QHBoxLayout();
    singleStartPointLayoutH2->addWidget(singleFreqLabel);
    singleStartPointLayoutH2->addWidget(singleFreqLineEdit);
    singleStartPointLayoutH2->addWidget(singleFreqUnitLabel);
    singleStartPointLayoutV1 = new QVBoxLayout();
    singleStartPointLayoutV1->addLayout(singleStartPointLayoutH1);
    singleStartPointLayoutV1->addLayout(singleStartPointLayoutH2);
    singlePointGroupBox->setLayout(singleStartPointLayoutV1);

    /* Groupbox for multiple start points */
    multiStartPointsGroupBox = new QGroupBox ("                                                           ", this) ;
    multiStartPointsRadioButton = new QRadioButton(tr("Advanced/Multiple start points"), multiStartPointsGroupBox);
    multiStartPointsRadioButton->move(14, -8);
    multiStartPointsRadioButton->setChecked(false);
    multiStartPointsRadioButton->setMinimumWidth(200);
    startPointRadioButtonGroup = new QButtonGroup(this);
    multiStartPointsLabelConfig = new QLabel(tr("Configure start points :"), this);
    multiStartPointsLabelConfig->setEnabled(false);
    multiStartPointsConfigButton = new QPushButton("...");
    multiStartPointsConfigButton->setMaximumWidth(50);
    multiStartPointsConfigButton->setEnabled(false);
    multiStartPointsLockButton = new QPushButton();
    multiStartPointsLockButton->setMaximumWidth(20);
    multiStartPointsLockButton->setIcon(QIcon(":/myResources/Resources/unlocked.png"));
    multiStartPointsLockButton->setEnabled(false);
    multiStartPointOnlyEndPointCheckBox = new QCheckBox(tr("Display only end points"));
    multiStartPointOnlyEndPointCheckBox->setEnabled(false);
    displayReturnLossCheckBox = new QCheckBox(tr("Display return loss chart"));
    displayReturnLossCheckBox->setEnabled(false);
    multiStartPointsLayoutH1 = new QHBoxLayout();
    multiStartPointsLayoutH1->addWidget(multiStartPointsLabelConfig);
    multiStartPointsLayoutH1->addWidget(multiStartPointsConfigButton);
    multiStartPointsLayoutH1->addWidget(multiStartPointsLockButton);
    multiStartPointLayoutV1 = new QVBoxLayout();
    multiStartPointLayoutV1->addLayout(multiStartPointsLayoutH1);
    multiStartPointLayoutV1->addWidget(multiStartPointOnlyEndPointCheckBox);
    multiStartPointLayoutV1->addWidget(displayReturnLossCheckBox);
    multiStartPointsGroupBox->setLayout(multiStartPointLayoutV1);

    startPointRadioButtonGroup->addButton(singlePointRadioButton);
    startPointRadioButtonGroup->setId(singlePointRadioButton,0);
    startPointRadioButtonGroup->addButton(multiStartPointsRadioButton);
    startPointRadioButtonGroup->setId(multiStartPointsRadioButton,1);

    /* Groupbox for target point */
    targetPointGroupBox = new QGroupBox (tr("Target point"), this);
    targetPointGroupBox->setCheckable(true);
    targetPointGroupBox->setChecked(true);
    targetPointImpedanceLabel = new QLabel(tr("Single point :"), this);
    targetPointRealImpedanceLabel = new QLabel("r", this);
    targetPointRealImpedanceLineEdit = new QLineEdit("50.0", this);
    targetPointImagImpedanceLabel = new QLabel("i", this);
    targetPointImagImpedanceLineEdit = new QLineEdit("0.0", this);
    targetPointLockButton = new QPushButton();
    targetPointLockButton->setMaximumWidth(20);
    targetPointLockButton->setIcon(QIcon(":/myResources/Resources/unlocked.png"));
    targetPointLayoutH1 = new QHBoxLayout();
    targetPointLayoutH1->addWidget(targetPointImpedanceLabel);
    targetPointLayoutH1->addWidget(targetPointRealImpedanceLabel);
    targetPointLayoutH1->addWidget(targetPointRealImpedanceLineEdit);
    targetPointLayoutH1->addWidget(targetPointImagImpedanceLabel);
    targetPointLayoutH1->addWidget(targetPointImagImpedanceLineEdit);
    targetPointLayoutH1->addWidget(targetPointLockButton);
    targetPointGroupBox->setLayout(targetPointLayoutH1);

    // Label 'Normal Z'
    chartImpedanceGroupBox = new QGroupBox(tr("Modify default Zo (50Ω)"), this);
    chartImpedanceGroupBox->setCheckable(true);
    chartImpedanceGroupBox->setChecked(false);
    chartImpedanceLabel = new QLabel(tr("Default Zo :"), this);
    chartImpedanceLineEdit = new QLineEdit("50.0", this);
    chartImpedanceUnitLabel = new QLabel("Ω", this);
    chartImpedanceLayoutH1 = new QHBoxLayout();
    chartImpedanceLayoutH1->addWidget(chartImpedanceLabel);
    chartImpedanceLayoutH1->addWidget(chartImpedanceLineEdit);
    chartImpedanceLayoutH1->addWidget(chartImpedanceUnitLabel);
    chartImpedanceGroupBox->setLayout(chartImpedanceLayoutH1);

    // Data page bottom widgets
    spacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);

    // Circuit page widgets
    circuitListView = new QListWidget(this);
    circuitListView->setContextMenuPolicy(Qt::CustomContextMenu);

    // Layout for data page //
    gridLayoutDataPage = new QGridLayout();
    gridLayoutDataPage->addWidget(singlePointGroupBox,          0, 0, 1, 4);
    gridLayoutDataPage->addWidget(multiStartPointsGroupBox,     1, 0, 1, 4);
    gridLayoutDataPage->addWidget(targetPointGroupBox,          2, 0, 1, 4);
    gridLayoutDataPage->addWidget(chartImpedanceGroupBox ,      3, 0, 1, 4);
    gridLayoutDataPage->addItem  (spacer,                       4, 0, 1, 4);

    // Layout for circuit page //
    layoutCircuitPage = new QVBoxLayout();
    layoutCircuitPage->addWidget(circuitListView);

    // The two tab
    dataPage = new QWidget(this);
    dataPage->setLayout(gridLayoutDataPage);
    circuitPage = new QWidget(this);
    circuitPage->setLayout(layoutCircuitPage);

    chartToolBox = new QTabWidget(this);
    chartToolBox->addTab(dataPage, tr("Chart settings"));
    chartToolBox->addTab(circuitPage, tr("Circuit edit/display"));

    splitter = new QSplitter(this);
    splitter->setOrientation(Qt::Horizontal);
    splitter->addWidget(m_smithChart);
    splitter->addWidget(chartToolBox);
    splitter->setChildrenCollapsible(false);
    chartToolBox->setMinimumWidth(250);
    chartToolBox->setMaximumWidth(400);
    splitter->setStretchFactor(0, 5); // m_smithChart is 5/8 in proportion
    splitter->setStretchFactor(1, 3); // chartToolBox is 3/8 in proportion

    setCentralWidget(splitter);

    // Top Menu
    //QAction *simulateAction = new QAction(tr("Force simulation"), this);
    //QAction *clearAction = new QAction(tr("Clear chart"), this);
    QAction *saveFileAction = new QAction(tr("Save"), this);
    saveFileAction->setShortcut(Qt::CTRL + Qt::Key_S);
    QAction *saveFileAsAction = new QAction(tr("Save as..."), this);
    QAction *openFileAction = new QAction(tr("Open..."), this);
    QAction *quitAction = new QAction(tr("Quit"), this);
    QMenu *menuFile = menuBar()->addMenu(tr("File"));
    menuFile->addAction(saveFileAction);
    menuFile->addAction(saveFileAsAction);
    menuFile->addAction(openFileAction);
    menuFile->addSeparator();
    menuFile->addAction(quitAction);

    forceFastSimAction = new QAction(tr("Force fast simulation"), this);
    forceFastSimAction->setCheckable(true);
    startSmoothRenderAction = new QAction(tr("Calculate an ultra-smooth render"), this);
    displayQCirclesAction = new QAction(tr("Display Q-Circles"), this);
    displayQCirclesAction->setCheckable(true);
    QMenu *menuDisplay = menuBar()->addMenu(tr("Display"));
    menuDisplay->addAction(displayQCirclesAction);
    menuDisplay->addSeparator();
    menuDisplay->addAction(forceFastSimAction);
    menuDisplay->addAction(startSmoothRenderAction);

    ENGAction = new QAction(tr("English"), this);
    FRAction = new QAction(tr("French"), this);
    QMenu *menuLang = menuBar()->addMenu(tr("Language"));
    menuLang->addAction(ENGAction);
    menuLang->addAction(FRAction);
    languageGroup = new QActionGroup(this);
    languageGroup->addAction(ENGAction);
    languageGroup->addAction(FRAction);
    ENGAction->setCheckable(true);
    FRAction->setCheckable(true);
    if (languageSelected==0)
        ENGAction->setChecked(true);
    else
        FRAction->setChecked(true);

    QAction *shortcutHelpAction= new QAction(tr("Shortcut list"), this);
    QAction *aboutAction= new QAction(tr("About"), this);
    QMenu *menuAbout = menuBar()->addMenu(tr("About"));
    menuAbout->addAction(shortcutHelpAction);
    menuAbout->addSeparator();
    menuAbout->addAction(aboutAction);

    // Bottom menu
    statusBar()->showMessage(tr("Ready"));

    chartToolBox->setCurrentIndex(0);

    qDebug() << "Drawing windows... OK";
    qDebug() << "Connecting signals...";

    // Connections
    connect(m_smithChart, SIGNAL(posUnderMouseUpdated()), this, SLOT(updateStatusBar()));
    connect(m_smithChart, SIGNAL(componentsListUpdated()), this, SLOT(updateCircuitDisplayWidget()));
    connect(m_smithChart, SIGNAL(startPointsUpdated()), this, SLOT(startPointsUpdateFromChartSlot()));
    connect(m_smithChart, SIGNAL(targetPointUpdated()), this, SLOT(targetPointUpdateFromChartSlot()));
    connect(m_smithChart, SIGNAL(startSim(SimQuality)), this, SLOT(simulationSlot(SimQuality)));
    connect(m_smithChart, SIGNAL(undoActionSignal()), this, SLOT(loadPreviousAutoSave()));
    connect(m_smithChart, SIGNAL(autoSaveSignal()), this, SLOT(autoSave()));

    connect(singleFreqLineEdit, SIGNAL(textChanged(QString)), this, SLOT(singleStartPointLineEditSlot()));
    connect(singleStartPointRealImpedanceLineEdit, SIGNAL(textEdited(QString)), this, SLOT(singleStartPointLineEditSlot()));
    connect(singleStartPointImagImpedanceLineEdit, SIGNAL(textEdited(QString)), this, SLOT(singleStartPointLineEditSlot()));
    connect(singleStartPointLockButton, SIGNAL(clicked(bool)), this, SLOT(singleStartPointsLockButtonSlot()));

    connect(startPointRadioButtonGroup, SIGNAL(buttonClicked(int)), this, SLOT(startPointRBGroupSlot(int)));
    connect(multiStartPointsConfigButton, SIGNAL(clicked()), this, SLOT(multipleStartPointsConfigButtonSlot()));
    connect(multiStartPointsLockButton, SIGNAL(clicked(bool)), this, SLOT(multipleStartPointsLockButtonSlot()));
    connect(multiStartPointOnlyEndPointCheckBox, SIGNAL(clicked(bool)), m_smithChart, SLOT(endPointsModeEnable(bool)));
    connect(displayReturnLossCheckBox, SIGNAL(clicked(bool)), this, SLOT(displayReturnLossCheckBoxClickedSlot(bool)));
    connect(m_returnLossChart, SIGNAL(rejected()), this, SLOT(displayReturnLossCheckBoxClosedSlot()));

    connect(targetPointGroupBox, SIGNAL(clicked(bool)), this, SLOT(targetPointCheckBoxClickedSlot(bool)));
    connect(targetPointRealImpedanceLineEdit, SIGNAL(textEdited(QString)), this, SLOT(targetPointLineEditSlot()));
    connect(targetPointImagImpedanceLineEdit, SIGNAL(textEdited(QString)), this, SLOT(targetPointLineEditSlot()));
    connect(targetPointLockButton, SIGNAL(clicked(bool)), this, SLOT(targetPointLockButtonSlot()));

    connect(chartImpedanceGroupBox, SIGNAL(clicked(bool)), this, SLOT(changeChartImpedanceCheckBoxSlot(bool)));
    connect(chartImpedanceLineEdit, SIGNAL(textEdited(QString)), this, SLOT(changeChartImpedanceSlot()));

    connect(chartToolBox, SIGNAL(currentChanged(int)), this, SLOT(updateCircuitDisplayWidget()) );
    connect(circuitListView, SIGNAL(doubleClicked(QModelIndex)) , this, SLOT(doubleClickInCompListSlot(QModelIndex)));
    connect(circuitListView,SIGNAL(customContextMenuRequested(const QPoint &)), this,SLOT(circuitListViewContextMenuRequested(const QPoint &)));


    connect(saveFileAction, SIGNAL(triggered()), this, SLOT(saveFileActionSlot()));
    connect(saveFileAsAction, SIGNAL(triggered()), this, SLOT(saveFileAsActionSlot()));
    connect(openFileAction, SIGNAL(triggered()), this, SLOT(openFileActionSlot()));
    connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
    connect(forceFastSimAction, SIGNAL(triggered(bool)), this, SLOT(forceFastSimActionSlot(bool)));
    connect(startSmoothRenderAction, SIGNAL(triggered()), this, SLOT(startSmoothRenderActionSlot()));
    connect(displayQCirclesAction, SIGNAL(triggered(bool)), this, SLOT(displayQCirclesActionSlot(bool)));
    connect(shortcutHelpAction, SIGNAL(triggered()), this, SLOT(shortcutHelpSlot()));
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(aboutSlot()));
    connect(languageGroup, SIGNAL(triggered(QAction*)), this, SLOT(languageSelectedSlot()));


    qDebug() << "Connecting signals... OK";
    qDebug() << "Setting frequency & start & target points...";

    singleStartPointLineEditSlot();
    targetPointLineEditSlot();
    changeChartImpedanceSlot();

    updateCircuitDisplayWidget();

    qDebug() << "Setting start & target points... OK";

    m_forcedFastSimulation = false;
    m_smithChart->setFocus(Qt::TabFocusReason);
    m_saveFileName = new QString();
    m_autoSaveEnable=true;
    autoSave();

    qDebug() << "-- MainWindow initialization finished. --";

    /// DEBUG
    //openFileActionSlot();
}


/* Interface window destructor */
MainWindow::~MainWindow()
{
    delete m_circuit;
    delete m_smithChart;
}





