
#include "returnLossChart.h"

ReturnLossChart::ReturnLossChart(QWidget *parent, Circuit &_circuit, QVector<complexFAndFreq> &_startPoints) : QDialog(parent), r_circuit(_circuit), r_startPoints(_startPoints)
{
    resize(800, 200);
    setWindowTitle("Return loss chart");

    /// Pop-up a bottom rigth of the screen
    QDesktopWidget desktop_widget;
    QRect mainScreenSize = desktop_widget.availableGeometry(desktop_widget.primaryScreen());
    int mainScreenHeight = mainScreenSize.height();
    int mainScreenWidth = mainScreenSize.width();

    this->move(mainScreenWidth - width()-20, mainScreenHeight - height()-60);

    qCustomPlot = new QCustomPlot();
    QHBoxLayout *centralLayout = new QHBoxLayout();
    centralLayout->addWidget(qCustomPlot);
    setLayout(centralLayout);

    connect(qCustomPlot, SIGNAL(mouseMove(QMouseEvent*)), this, SLOT(showPointToolTip(QMouseEvent*)));

}

void ReturnLossChart::showPointToolTip(QMouseEvent *event)
{
    double minDist=-1;
    int indexMinDist=0;

    for (int i=0 ; i<m_displayedPointsX.size() ; i++)
    {
        QPointF posPoint(qCustomPlot->xAxis->coordToPixel(m_displayedPointsX[i]),     qCustomPlot->yAxis->coordToPixel(m_displayedPointsY[i]));
        QPointF posMouse(event->pos().x(),   event->pos().y());
        double dist = (posPoint-posMouse).manhattanLength();
        if (i==0){
            minDist=dist;
            indexMinDist=0;
        }
        else {
            if (dist<minDist) {
                minDist=dist;
                indexMinDist=i;
            }
        }
    }

    if (minDist<12)
        setToolTip(QString("%1 , %2").arg(m_displayedPointsX[indexMinDist]).arg(m_displayedPointsY[indexMinDist]));
    else
        setToolTip(QString(""));
}

void ReturnLossChart::paintEvent(QPaintEvent * /* the event */)
{
    complexF lastDataPoint;
    complexF reflectionCoeff;
    complexF reflectionCoeffPolar;
    double returnLoss, minReturnLoss=-20.0;
    m_displayedPointsX.clear();
    m_displayedPointsX.resize(r_startPoints.size());
    m_displayedPointsY.clear();
    m_displayedPointsY.resize(r_startPoints.size());


    qDebug() << "Paint event return loss chart";
    //painter = new QPainter(this);

    for (int numStartPoint=0 ; numStartPoint<r_startPoints.size() ; numStartPoint++)   // each start points
    {
        lastDataPoint = r_circuit.getComponentDataPoints(r_circuit.getSize()-1).at(numStartPoint).last();
        reflectionCoeff = ZloadToCoeffReflect(lastDataPoint, 1.0);
        reflectionCoeffPolar = cartToPolar(reflectionCoeff);
        returnLoss = 20.0*log10(reflectionCoeffPolar.real());

        m_displayedPointsX[numStartPoint] = r_startPoints[numStartPoint].freq;
        m_displayedPointsY[numStartPoint] = returnLoss;

        if (numStartPoint == 0)
            minReturnLoss = returnLoss;
        else {
            if (returnLoss < minReturnLoss)
                minReturnLoss = returnLoss;
        }
    }

    // Create graph and assign data to it
    qCustomPlot->addGraph();
    qCustomPlot->graph(0)->setData(m_displayedPointsX, m_displayedPointsY);
    qCustomPlot->graph(0)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssDisc , Qt::blue, Qt::red, 6));
    // Give the axes some labels:
    qCustomPlot->xAxis->setLabel("Freq (GHz)");
    qCustomPlot->yAxis->setLabel("Return Loss (dB)");
    qCustomPlot->setInteractions(QCP::iSelectPlottables);
    // Set axes ranges, so we see all data:
    //qDebug() << "Min return loss :" << minReturnLoss;
    //qDebug() << m_displayedPointsX;
    //qDebug() << m_displayedPointsY;

    if (r_startPoints.size() >=2)
    {
        qCustomPlot->xAxis->setRange(m_displayedPointsX.first()*0.99, m_displayedPointsX.last()*1.01);
        qCustomPlot->xAxis->setAutoTickStep(false);
        qCustomPlot->xAxis->setTickStep(m_displayedPointsX[1]-m_displayedPointsX[0]);
        qCustomPlot->yAxis->setAutoTickStep(false);
        if (minReturnLoss < -39.0) {
            qCustomPlot->yAxis->setRange(-50.0, 0.0);
            qCustomPlot->yAxis->setTickStep(10.0);
        }
        else if (minReturnLoss < -29.0) {
            qCustomPlot->yAxis->setRange(-40.0, 0.0);
            qCustomPlot->yAxis->setTickStep(10.0);
        }
        else if (minReturnLoss < -19.0) {
            qCustomPlot->yAxis->setRange(-30.0, 0.0);
            qCustomPlot->yAxis->setTickStep(5.0);
        }
        else {
            qCustomPlot->yAxis->setRange(-20.0, 0.0);
            qCustomPlot->yAxis->setTickStep(5.0);
        }
    }
    else
    {
        qCustomPlot->xAxis->setRange(m_displayedPointsX.first()*0.9, m_displayedPointsX.last()*1.1);
        qCustomPlot->yAxis->setRange(m_displayedPointsY.first()-10.0, 0.0);
    }
    qCustomPlot->replot();
    //QPixmap myPlot = qCustomPlot->toPixmap(width(), height());
    //painter->drawPixmap(QPointF(0, 0) , myPlot);

}


