/****************************************************************************
 * QSmithTool :                                                             *
 *   Copyright (C) 2015 by Sébastien Lissarre                               *
 *   sebastien.lissarre@gmail.com                                           *
 *                                                                          *
 * Smith Chart drawing base :                                               *
 *   Copyright (C) 2008 by Lisandro Damián Nicanor Pérez Meyer              *
 *   perezmeyer@gmail.com                                                   *
 *                                                                          *
 *                                                                          *
 *  This file is part of QSmithTool.                                        *
 *                                                                          *
 *  QSmithTool is free software: you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  QSmithTool is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with QSmithTool.  If not, see <http://www.gnu.org/licenses/>.     *
 ***************************************************************************/



#ifndef RETURNLOSSCHART_H
#define RETURNLOSSCHART_H

#include <QDialog>
#include <QLabel>
#include <QVector>
#include <QRect>

#include "rfFctsLibrary.h"
#include "circuit.h"
#include "qcustomplot.h"

/**
This class dislay return losses vs freq. of the simulated circuit, using a qcustomplot widget
*/
class ReturnLossChart : public QDialog
{
    Q_OBJECT

public:
    ReturnLossChart(QWidget *parent, Circuit &_circuit, QVector<complexFAndFreq> &_startPoints);

private slots:
    void showPointToolTip(QMouseEvent *event);

protected:
    void paintEvent(QPaintEvent *event);

private:

    /// Reference to the circuit
    Circuit &r_circuit;

    /// Reference to the array of start points (impedance & frequency)
    QVector<complexFAndFreq> &r_startPoints;

    /// QPainter to draw the chart
    QPainter *painter;

    /// The chart (QCustomPlot library)
    QCustomPlot *qCustomPlot;

    QVector<double> m_displayedPointsX;
    QVector<double> m_displayedPointsY;

};

#endif



