
#include "rfFctsLibrary.h"


complexFAndFreq complexFAndFreqInitializer() {
   static complexFAndFreq zeroInitialized;
   zeroInitialized.point = complexF(0.0,0.0);
   zeroInitialized.freq = 1.0;
   return zeroInitialized;
}


/// Check for a valid impedance real part
bool checkRealPartImpValid(double value) {
    return (value >= 0.0) ? true : false;
}
bool checkRealPartImpValid(QString strValue, double *result)
{
    double tempVal;
    bool convSuccess=0;

    tempVal = strValue.toDouble(&convSuccess);

    if ((convSuccess==true) && (checkRealPartImpValid(tempVal))) {
        *result = tempVal;
        return true;
    }
    else
        return false;
}

/// Check for a valid impedance imag part
bool checkImagPartImpValid(double) {
    return true;
}
bool checkImagPartImpValid(QString strValue, double *result)
{
    double tempVal;
    bool convSuccess=0;

    tempVal = strValue.toDouble(&convSuccess);

    if ((convSuccess==true) && (checkImagPartImpValid(tempVal))) {
        *result = tempVal;
        return true;
    }
    else
        return false;
}

/// Check for a valid frequency value
bool checkFrequencyValid(double value) {
    return (value > 0.0) ? true : false;
}
bool checkFrequencyValid(QString strValue, double *result) {
    double tempVal;
    bool convSuccess=0;

    tempVal = strValue.toDouble(&convSuccess);

    if ((convSuccess==true) && (checkFrequencyValid(tempVal))) {
        *result = tempVal;
        return true;
    }
    else
        return false;
}


complexF polarToCart(complexF polarCplx)
{
    complexF result;
    result.real(polarCplx.real()*cos(polarCplx.imag()*_PI/180.0));
    result.imag(polarCplx.real()*sin(polarCplx.imag()*_PI/180.0));
    return result;
}
QPointF polarToCartXY(double magn, double angle)
{
    QPointF result ( magn*cos(angle*_PI/180.0)  ,  magn*sin(angle*_PI/180.0) );
    return result;
}

complexF cartToPolar(complexF cartCplx)
{
    complexF result;
    result.real(abs(cartCplx)); // the modulus of a complex is found with abs() and not norm()
    result.imag(arg(cartCplx)/_PI*180.0);
    return result;
}


complexF coeffReflectToZload(complexF coeffReflect, double Z0)
{
    //qDebug() << "Convert Reflect " << coeffReflect.real() << coeffReflect.imag() << "to ZLoad with Z0=" << Z0;
    complexF result = Z0 * ((1.0+coeffReflect)/(1.0-coeffReflect));
    //qDebug() << "Result = " << result.real() << result.imag();
    return result;
}

complexF ZloadToCoeffReflect(complexF Zload, double Z0)
{
    complexF result = (Zload-Z0)/(Zload+Z0);
    return result;
}

complexF ZloadToCoeffReflectPolar(complexF Zload, double Z0)
{
    complexF result = cartToPolar((Zload-Z0)/(Zload+Z0));
    return result;
}

double parameterNormToDenormConv(double denormParam, ComponentType type, int numParam, double Z0)
{
    double resNormParam;

    if ((type==CAPA_S) || (type==CAPA_P))
        resNormParam = denormParam / Z0;
    else if ((type==SELF_S) || (type==SELF_P) || (type==RESIST_S) || (type==RESIST_P))
        resNormParam = denormParam * Z0;
    else if (  ((type==LINE_S) || (type==LINE_OPEN) || (type==LINE_SHORT)) && numParam==1) // line impedance
        resNormParam = denormParam * Z0;
    else
        resNormParam = denormParam;

    return resNormParam;
}
double parameterDenormToNormConv(double normParam, ComponentType type, int numParam, double Z0)
{
    double resDenormParam;

    if ((type==CAPA_S) || (type==CAPA_P))
        resDenormParam = normParam * Z0;
    else if ((type==SELF_S) || (type==SELF_P) || (type==RESIST_S) || (type==RESIST_P))
        resDenormParam = normParam / Z0;
    else if (  ((type==LINE_S) || (type==LINE_OPEN) || (type==LINE_SHORT)) && numParam==1) // line impedance
        resDenormParam = normParam / Z0;
    else
        resDenormParam = normParam;

    return resDenormParam;
}


QString complexNumberToQString(complexF point)
{
    QString res="";
    double valR = point.real();
    double valI = point.imag();

    // Now : res=""
    if (QString::number(valR).size()>3)
        res += QString::number(valR, 'f', 3); // limit the number of decimals...
    else
        res += QString::number(valR);

    // Now : res="2.3"
    if (valI<0.0)
        res += QString("-j*");
    else
        res += QString("+j*");

    // Now : res="2.3-j*"
    if (QString::number(valR).size()>3)
        res += QString::number(abs(valI), 'f', 3); // limit the number of decimals ...
    else
        res += QString::number(abs(valI));

    // Now : res="2.3-j*4.5"
    return res;
}

QString doubleToQString(double value, int nbDecimals)
{
    QStringList splitNumber = QString::number(value).split('.');
    if (splitNumber.size()==2) // non integer value
    {
        if (QString::number(value).size()>nbDecimals)
            return (QString::number(value, 'f', nbDecimals)); // limit the number of decimals...
        else
            return (QString::number(value));
    }
    else // integer value => add ".0" at the end
    {
        return (QString::number(value) + QString(".0"));
    }
}

/*
complexF myABCDtoS11(QVector<complexF> ABCDmatrix) {
    complexF S11;
    S11 = (ABCDmatrix[0] + ABCDmatrix[1] - ABCDmatrix[2] - ABCDmatrix[3]) / (ABCDmatrix[0] + ABCDmatrix[1] + ABCDmatrix[2] + ABCDmatrix[3]);
    return S11;
}

double myS11toS11dB(complexF S11) {
    double S11dB = (20.0*log10(sqrt(S11.real()*S11.real()+S11.imag()*S11.imag())));
    return S11dB;
}
*/

double myABCDtoS21dB(QVector<complexF> ABCDmatrix, complexF Zout) {
    complexF S21;
    S21 = 2.0*sqrt(Zout.real()) / (ABCDmatrix[0]*Zout + ABCDmatrix[1] + ABCDmatrix[2]*Zout + ABCDmatrix[3]);
    return (20.0*log10(sqrt(S21.real()*S21.real()+S21.imag()*S21.imag())));
}

QVector<complexF> matrixProduct(QVector<complexF> ABCDmatrix1, QVector<complexF> ABCDmatrix2) {
    QVector<complexF> matrixABCDres(4);
    matrixABCDres[0]=ABCDmatrix1[0]*ABCDmatrix2[0]+ABCDmatrix1[1]*ABCDmatrix2[2];
    matrixABCDres[1]=ABCDmatrix1[0]*ABCDmatrix2[1]+ABCDmatrix1[1]*ABCDmatrix2[3];
    matrixABCDres[2]=ABCDmatrix1[2]*ABCDmatrix2[0]+ABCDmatrix1[3]*ABCDmatrix2[2];
    matrixABCDres[3]=ABCDmatrix1[2]*ABCDmatrix2[1]+ABCDmatrix1[3]*ABCDmatrix2[3];
    return matrixABCDres;
}
