/****************************************************************************
 * QSmithTool :                                                             *
 *   Copyright (C) 2015 by Sébastien Lissarre                               *
 *   sebastien.lissarre@gmail.com                                           *
 *                                                                          *
 * Smith Chart drawing base :                                               *
 *   Copyright (C) 2008 by Lisandro Damián Nicanor Pérez Meyer              *
 *   perezmeyer@gmail.com                                                   *
 *                                                                          *
 *                                                                          *
 *  This file is part of QSmithTool.                                        *
 *                                                                          *
 *  QSmithTool is free software: you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  QSmithTool is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with QSmithTool.  If not, see <http://www.gnu.org/licenses/>.     *
 ***************************************************************************/

#ifndef RFMATHLIB_H
#define RFMATHLIB_H

#include "defines.h"
#include <QString>
#include <QPointF>
#include <QStringList>
#include <cmath>
#include <QDebug>


#define NB_COMP_TYPE 10
enum ComponentType { CAPA_S, SELF_S, RESIST_S, LINE_S, CAPA_P, SELF_P, RESIST_P, LINE_OPEN, LINE_SHORT, NOT_DEF };
static const QString compName[NB_COMP_TYPE]     = {"Serial capacitor", "Serial inductor", "Serial resistor", "Serial tr. line","Parallel capacitor","Parallel inductor","Parallel resistor","Open tr. line","Short tr. line", "Not defined"};
static const QString compStrUnit[NB_COMP_TYPE]  = {"pF", "nH", "Ω", "mm", "pF", "nH", "Ω", "mm", "mm", ""};

/// An initiliazer function for 'complexFAndFreq' type
complexFAndFreq complexFAndFreqInitializer();
/// Return true if the value is a valid real part for a complex impedance
bool checkRealPartImpValid(double value);
bool checkRealPartImpValid(QString strValue, double *result);
/// Return true if the value is a valid imaginary part for a complex impedance
bool checkImagPartImpValid(double);
bool checkImagPartImpValid(QString strValue, double *result);
/// Return true if the value is a valid frequency
bool checkFrequencyValid(double value);
bool checkFrequencyValid(QString strValue, double *result);

/// Polar to cartesian coord conversion
complexF polarToCart(complexF polarCplx);
/// Polar to cartesian coord (QPointF format) conversion
QPointF polarToCartXY(double magn, double angle);
/// Cartesian to polar coord conversion
complexF cartToPolar(complexF cartCplx);

/// Convert a reflection coeff (cartesian) to an unnormalized impedance (like 50.0+j*3.1)
complexF coeffReflectToZload(complexF coeffReflect, double Z0);
/// Convert a not normalized impedance (like 50.0+j*3.1) to reflection coeff (cartesian)
complexF ZloadToCoeffReflect(complexF Zload, double Z0);
/// Convert a not normalized impedance (like 50.0+j*3.1) to reflection coeff (polar)
complexF ZloadToCoeffReflectPolar(complexF Zload, double Z0);

/// Convert a normalized (1 ohm chart) component parameter to denormalized value (like in 50ohm chart)
double parameterNormToDenormConv(double denormParam, ComponentType type, int numParam, double Z0);
/// Same operation inverted
double parameterDenormToNormConv(double normParam, ComponentType type, int numParam, double Z0);

/// Return "2.32 - j* 3.2" when arg=QPointF(2.32,-3.2)
QString complexNumberToQString(complexF point);
QString doubleToQString(double value, int nbDecimals=3);

/// convert ABCD matrix to S11 parameter (complex) and S11dB (dB)
//complexF myABCDtoS11(QVector<complexF> ABCDmatrix);
//double myS11toS11dB(complexF S11);
/// convert ABCD matrix to S21 parameter (in dB)
double myABCDtoS21dB(QVector<complexF> ABCDmatrix, complexF Zout);
/// calc matrix product of ABCD1 and ABCD2
QVector<complexF> matrixProduct(QVector<complexF> ABCDmatrix1, QVector<complexF> ABCDmatrix2);


#endif



